#ifndef __B2_POLYGON_SHAPE_H__
#define __B2_POLYGON_SHAPE_H__

/// Create b2PolygonShape
b2PolygonShape *b2PolygonShape_Create();

/// Destroy b2PolygonShape
void b2PolygonShape_Destroy(b2PolygonShape *shape);

/// Implement b2Shape.
b2Shape* b2PolygonShape_Clone(b2PolygonShape *shape, b2BlockAllocator* allocator);

/// Copy vertices. This assumes the vertices define a convex polygon.
/// It is assumed that the exterior is the the right of each edge.
void b2PolygonShape_Set(b2PolygonShape *shape, const VECTOR *vertices, var vertexCount);

/// Build vertices to represent an axis-aligned box.
/// @param hx the half-width.
/// @param hy the half-height.
void b2PolygonShape_SetAsBox1(b2PolygonShape *shape, var hx, var hy);

/// Build vertices to represent an oriented box.
/// @param hx the half-width.
/// @param hy the half-height.
/// @param center the center of the box in local coordinates.
/// @param angle the rotation of the box in local coordinates.
void b2PolygonShape_SetAsBox2(b2PolygonShape *shape, var hx, var hy, const VECTOR *center, var angle);

/// Set this as a single edge.
void b2PolygonShape_SetAsEdge(b2PolygonShape *shape, const VECTOR *v1, const VECTOR *v2);

/// @see b2Shape::TestPoint
var b2PolygonShape_TestPoint(b2PolygonShape *shape, const b2Transform *transform, const VECTOR *p);

/// Implement b2Shape.
var b2PolygonShape_RayCast(b2PolygonShape *shape, b2RayCastOutput* output, const b2RayCastInput *input, const b2Transform *transform);

/// @see b2Shape::ComputeAABB
void b2PolygonShape_ComputeAABB(b2PolygonShape *shape, b2AABB* aabb, const b2Transform *transform);

/// @see b2Shape::ComputeMass
void b2PolygonShape_ComputeMass(b2PolygonShape *shape, b2MassData* massData, var density);

/// Get the supporting vertex index in the given direction.
var b2PolygonShape_GetSupport(b2PolygonShape *shape, const VECTOR *d);

/// Get the supporting vertex in the given direction.
void b2PolygonShape_GetSupportVertex(b2PolygonShape *shape, const VECTOR *d, VECTOR *outSupport);

/// Get the vertex count.
var b2PolygonShape_GetVertexCount(b2PolygonShape *shape);

/// Get a vertex by index.
void b2PolygonShape_GetVertex(b2PolygonShape *shape, var index, VECTOR *outVertex);

/// Set m_centroid property
void b2PolygonShape_SetCentroid(b2PolygonShape *shape, const VECTOR *centroid);

/// Get m_centroid property
void b2PolygonShape_GetCentroid(b2PolygonShape *shape, VECTOR *outCentroid);

/// Set vertex at index
void b2PolygonShape_SetVertex(b2PolygonShape *shape, const VECTOR *vertex, var index);

/// Get vertex at index
/*void b2PolygonShape_GetVertex(b2PolygonShape *shape, VECTOR *outVertex, var index);*/

/// Set normal at index
void b2PolygonShape_SetNormal(b2PolygonShape *shape, const VECTOR *normal, var index);

/// Get normal at index
void b2PolygonShape_GetNormal(b2PolygonShape *shape, VECTOR *outNormal, var index);

#endif //__B2_POLYGON_SHAPE_H__
