#ifndef __B2_AABB_H__
#define __B2_AABB_H__

/// \brief Create b2AABB instance.
///
/// @return b2AABB instance that can be used with box2D functions that expect b2AABB instance.
b2AABB *b2AABB_Create();

/// \brief Delete b2AABB instance.
///
/// @param aabb b2AABB instance to delete.
void b2AABB_Destroy(b2AABB *aabb);

/// \brief Verify that the bounds are sorted.
///
/// @param aabb b2AABB instance.
var b2AABB_IsValid(b2AABB *aabb);

/// \brief Get the center of the AABB.
///
/// @param aabb b2AABB instance.
/// @param outCenter pre allocated VECTOR struct to receive the b2AABB center.
void b2AABB_GetCenter(b2AABB *aabb, VECTOR *outCenter);

/// \brief Get the extents of the AABB (half-widths).
///
/// @param aabb b2AABB instance.
/// @param outExtents pre-allocated VECTOR struct to receive th b2AABB extents.
void b2AABB_GetExtents(b2AABB *aabb, VECTOR *outExtents);

/// \brief Combine two AABBs into this one.
///
/// @param aabb b2AABB instance.
/// @param aabb1 second b2AABB instance to combine with the first b2AABB into the first.
/// @param aabb2 third b2AABB instance to combine with the second into the first.
void b2AABB_Combine(b2AABB *aabb, const b2AABB *aabb1, const b2AABB *aabb2);

/// \brief Does this aabb contain the provided AABB.
///
/// @param aabb b2AABB instance.
/// @param aabb1 b2AABB instance to be tested if contained inside the first.
var b2AABB_Contains(b2AABB *aabb, const b2AABB *aabb1);

/// \brief b2AABB RayCast
///
/// Do RayCast test with the b2AABB, depending on the b2RayCastInput input.
/// The result will be stored in the output b2RayCastOutput struct.
/// @param aabb b2AABB instance.
/// @param output A pre allocated b2RayCastOutput instance to receive the result.
/// @param input b2RayCastInput instance, containing the test parameters.
var b2AABB_RayCast(b2AABB *aabb, b2RayCastOutput* output, const b2RayCastInput *input);

/// \brief Set LowerBound Vector
///
/// @param aabb b2AABB instance.
/// @param lowerBound The desired lower bound.
void b2AABB_SetLowerBound(b2AABB *aabb, const VECTOR *lowerBound);

/// \brief Get LowerBound Vector
///
/// @param aabb b2AABB instance.
/// @param outLowerBound A pre initialized VECTOR struct to receive the b2AABB actual lower bound
void b2AABB_GetLowerBound(b2AABB *aabb, VECTOR *outLowerBound);

/// \brief Set UpperBound Vector
///
/// @param aabb b2AABB instance.
/// @param lowerBound The desired upper bound.
void b2AABB_SetUpperBound(b2AABB *aabb, const VECTOR *upperBound);

/// \brief Get UpperBound Vector
///
/// @param aabb b2AABB instance.
/// @param outLowerBound A pre initialized VECTOR struct to receive the b2AABB actual upper bound
void b2AABB_GetUpperBound(b2AABB *aabb, VECTOR *outUpperBound);

#endif //__B2_AABB_H__
