#ifndef __B2_COLLISION_H__
#define __B2_COLLISION_H__

/// \brief Compute the collision manifold between two circles.
void b2CollideCircles(b2Manifold* manifold, const b2CircleShape* circle1, const b2Transform *xf1, const b2CircleShape* circle2, const b2Transform *xf2);

/// \brief Compute the collision manifold between a polygon and a circle.
void b2CollidePolygonAndCircle(b2Manifold* manifold, const b2PolygonShape* polygon, const b2Transform *xf1, const b2CircleShape* circle, const b2Transform *xf2);

/// \brief Compute the collision manifold between two polygons.
void b2CollidePolygons(b2Manifold* manifold, const b2PolygonShape* polygon1, const b2Transform *xf1, const b2PolygonShape* polygon2, const b2Transform *xf2);

/// \brief Clipping for contact manifolds.
var b2ClipSegmentToLine(b2ClipVertex **vOut, const b2ClipVertex **vIn, const VECTOR *normal, var offset);

/// \brief Compute the point states given two manifolds.
///
/// The states pertain to the transition from manifold1
/// to manifold2. So state1 is either persist or remove while state2 is either add or persist.
void b2GetPointStates(var *state1, var *state2, const b2Manifold* manifold1, const b2Manifold* manifold2);

/// \brief Determine if two generic shapes overlap.
var b2Testb2ShapeOverlap(const b2Shape* shapeA, const b2Shape* shapeB, const b2Transform *xfA, const b2Transform *xfB);

/// \brief Test b2AABB overlap
var b2Testb2AABBOverlap(const b2AABB *a, const b2AABB *b);

#endif //__B2_COLLISION_H__
