#ifndef __B2_DISTANCE_PROXY_H__
#define __B2_DISTANCE_PROXY_H__

/// \brief Create a DistanceProxy instance
///
/// A distance proxy is used by the GJK algorithm.
/// It encapsulates any shape
b2DistanceProxy *b2DistanceProxy_Create();

/// \brief Destroy the DistanceProxy provided as parameter
///
/// @param proxy The DistanceProxy to be destroyed
void b2DistanceProxy_Destroy(b2DistanceProxy *proxy);

/// \brief Initialize the proxy using the given shape.
///
/// The shape must remain in scope while the proxy is in use.
/// @param proxy The b2DistanceProxy
/// @param shape The shape to be used (must remain in scope)
void b2DistanceProxy_Set(b2DistanceProxy *proxy, const b2Shape *shape);

/// \brief Get the supporting vertex index in the given direction.
///
/// @param proxy The b2DistanceProxy
/// @param d The direction vector
/// @return The supporting vector index
var b2DistanceProxy_GetSupport(b2DistanceProxy *proxy, const VECTOR *d);

/// \brief Get the supporting vertex in the given direction.
///
/// @param proxy The b2DistanceProxy
/// @param d The direction vector
/// @param outVertex The resulting vector
void b2DistanceProxy_GetSupportVertex(b2DistanceProxy *proxy, const VECTOR *d, VECTOR *outVertex);

/// \brief Get the vertex count.
///
/// @param proxy The b2DistanceProxy
/// @return The vertex count
var b2DistanceProxy_GetVertexCount(b2DistanceProxy *proxy);

/// Get a vertex by index. Used by b2Distance.
/// @param proxy The b2DistanceProxy
/// @param index The vertex index
/// @param outVertex The resulting vectex position
void b2DistanceProxy_GetVertex(b2DistanceProxy *proxy, var index, VECTOR *outVertex);

#endif //__B2_DISTANCE_PROXY_H__
