#ifndef __B2_MANIFOLD_H__
#define __B2_MANIFOLD_H__

/// \brief Create a b2Manifold Instance
b2Manifold *b2Manifold_Create();

/// \brief Destroy b2manifold
void b2Manifold_Destroy(b2Manifold *manifold);

/// \brief Set b2Manifold local Normal
///
/// @param manifold The manifold instance
/// @param normal The new normal value
void b2Manifold_SetLocalNormal(b2Manifold *manifold,const VECTOR *normal);

/// \brief Get b2Manifold local Normal
///
/// @param manifold The manifold instance
/// @param outNormal A preinitialized vector to store the result
void b2Manifold_GetLocalNormal(b2Manifold *manifold, VECTOR *outNormal);

/// \brief Set b2Manifold local Point
///
/// @param manifold The manifold instance
/// @param point The new local point value
void b2Manifold_SetLocalPoint(b2Manifold *manifold,const VECTOR *point);

/// \brief Get b2Manifold local Point
///
/// @param manifold The manifold instance
/// @param outPoint A preinitialized vector to store the result
void b2Manifold_GetLocalPoint(b2Manifold *manifold, VECTOR *outPoint);

/// \brief Set b2Manifold type
///
/// @param manifold The manifold instance
/// @param type The new type
void b2Manifold_SetType(b2Manifold *manifold, var type);

/// \brief Get b2Manifold Type
///
/// @param manifold The manifold instance
/// @return var the manifold type
var b2Manifold_GetType(b2Manifold *manifold);

/// \brief Set b2Manifold pointCount
///
/// @param manifold The manifold instance
/// @param type The new pointCount
void b2Manifold_SetPointCount(b2Manifold *manifold, var pointCount);

/// \brief Get b2Manifold pointCount
///
/// @param manifold The manifold instance
/// @return var the manifold type
var b2Manifold_GetPointCount(b2Manifold *manifold);


/// \brief Set all b2Manifold points
///
/// @param manifold The b2Manifold instance
/// @param points An array of 2 b2ManifoldPoint pointers containing the new values
void b2Manifold_SetPoints(b2Manifold *manifold,const b2ManifoldPoint **points);

/// \brief Get all b2Manifold points
///
/// @param manifold The b2Manifold instance
/// @param outPoints An array of 2 preintialized b2ManifoldPoint pointers to receive the result
void b2Manifold_GetPoints(b2Manifold *manifold, b2ManifoldPoint **outPoints);

/// \brief Set b2Manifold point at index
///
/// @param manifold The b2Manifold instance
/// @param point A b2ManifoldPoint pointer containing the new value
/// @param index Point index
/// @return 1 on success and 0 on failure
var b2Manifold_SetPoint(b2Manifold *manifold, const b2ManifoldPoint *point, var index);

/// \brief Get b2Manifold point at index
///
/// @param manifold The b2Manifold instance
/// @param outPoint A preintialized b2ManifoldPoint pointer to receive the result
/// @param index Point index
/// @return 1 on success and 0 on failure
var b2Manifold_GetPoint(b2Manifold *manifold, b2ManifoldPoint *outPoint, var index);

#endif //__B2_MANIFOLD_H__
