#ifndef __B2_WORLD_MANIFOLD_H__
#define __B2_WORLD_MANIFOLD_H__

/// \brief Create 
b2WorldManifold *b2WorldManifold_Create();

/// \brief Destroy b2WorldManifold
void b2WorldManifold_Destroy(b2WorldManifold *worldManifold);

/// \brief Evaluate the manifold with supplied transforms.
/// This assumes modest motion from the original state. This does not change the
/// point count, impulses, etc. The radii must come from the shapes
/// that generated the manifold.
void b2WorldManifold_Initialize(b2WorldManifold *worldManifold, const b2Manifold* manifold, const b2Transform *xfA, var radiusA, const b2Transform *xfB, var radiusB);

/// \brief Set world manifold normal
///
/// world vector pointing from A to B
/// @param worldManifold The world manifold
/// @param normal The new normal value
void  b2WorldManifold_SetNormal(b2WorldManifold *worldManifold, const VECTOR *normal);

/// \brief Get world manifold normal
///
/// world vector pointing from A to B
/// @param worldManifold The world manifold
/// @param outNormal A preinitialized vector to receive the value
void  b2WorldManifold_GetNormal(b2WorldManifold *worldManifold, VECTOR *outNormal);

/// \brief Set all b2Manifold points
///
/// world contact point (point of intersection)
/// @param manifold The b2Manifold instance
/// @param points An array of 2 vector containing the new values
void b2WorldManifold_SetPoints(b2WorldManifold *worldManifold, const VECTOR *points);

/// \brief Get all b2Manifold points
///
/// world contact point (point of intersection)
/// @param manifold The b2Manifold instance
/// @param outPoints An array of 2 Vectors to receive the result
void b2WorldManifold_GetPoints(b2WorldManifold *worldManifold, VECTOR *outPoints);

/// \brief Set b2Manifold point at index
///
/// world contact point (point of intersection)
/// @param manifold The b2Manifold instance
/// @param point A VECTOR containing the new value
/// @param index Point index
/// @return 1 on success and 0 on failure
var b2WorldManifold_SetPoint(b2WorldManifold *worldManifold, const VECTOR *point, var index);

/// \brief Get b2Manifold point at index
///
/// world contact point (point of intersection)
/// @param manifold The b2Manifold instance
/// @param outPoint A preintialized VECTOR to receive the result
/// @param index Point index
/// @return 1 on success and 0 on failure
var b2WorldManifold_GetPoint(b2WorldManifold *worldManifold, VECTOR *outPoint, var index);

#endif //__B2_WORLD_MANIFOLD_H__
