#ifndef __B2_SWEEP_H__
#define __B2_SWEEP_H__

/// \brief Create a b2Sweep instance
b2Sweep *b2Sweep_Create();

/// \brief Destroy the b2Sweep passed as parameter
///
/// @param sweep The weep instance to destroy
void b2Sweep_Destroy(b2Sweep *sweep);

/// \brief Get the interpolated transform at a specific time.
///
/// @param sweep The sweep instance
/// @param xf a preInitialized transform to put the result into
/// @param alpha is a factor in [0,1], where 0 indicates t0.
void b2Sweep_GetTransform(b2Sweep *sweep, b2Transform* xf, var alpha);

/// \brief Advance the sweep forward, yielding a new initial state.
///
/// @param sweep The sweep instance
/// @param t the new initial time.
void b2Sweep_Advance(b2Sweep *sweep, var t);

/// \brief Normalize the angles.
void b2Sweep_Normalize(b2Sweep *sweep);

/// \brief Set b2Sweep localCenter 
///
/// local center of mass position
/// @param sweep The b2Sweep instance
/// @param localCenter The new localCenter
void b2Sweep_SetLocalCenter(b2Sweep *sweep,const VECTOR *localCenter);

/// \brief Get b2Sweep localCenter 
///
/// local center of mass position
/// @param sweep The b2Sweep instance
/// @param outCenter A preinitialized vector to set the localCenter into
void b2Sweep_GetLocalCenter(b2Sweep *sweep, VECTOR *outCenter);

/// \brief Set b2Sweep c0 
///
/// center world positions
/// @param sweep The b2Sweep instance
/// @param c0 The new c0 value
void b2Sweep_SetC0(b2Sweep *sweep, const VECTOR *c0);

/// \brief Get b2Sweep c0 
///
/// center world positions
/// @param sweep The b2Sweep instance
/// @param outC0 A preinitialized vector to set the current c0 value into
void b2Sweep_GetC0(b2Sweep *sweep, VECTOR *outC0);

/// \brief Set b2Sweep c 
///
/// center world positions
/// @param sweep The b2Sweep instance
/// @param c The new c value
void b2Sweep_SetC(b2Sweep *sweep, const VECTOR *c);

/// \brief Get b2Sweep c0 
///
/// center world positions
/// @param sweep The b2Sweep instance
/// @param outC A preinitialized vector to set the current c0 value into
void b2Sweep_GetC(b2Sweep *sweep, VECTOR *outC);

/// \brief Set b2Sweep a0
///
/// world angles
/// @param sweep The b2Sweep instance
/// @param a0 The new a0 value
void b2Sweep_SetA0(b2Sweep *sweep, var a0);

/// \brief Get b2Sweep a0 
///
/// world angles
/// @param sweep The b2Sweep instance
/// @return the current a0 value
var b2Sweep_GetA0(b2Sweep *sweep);

/// \brief Set b2Sweep a
///
/// world angles
/// @param sweep The b2Sweep instance
/// @param a The new a value
void b2Sweep_SetA(b2Sweep *sweep, var a);

/// \brief Get b2Sweep a 
///
/// world angles
/// @param sweep The b2Sweep instance
/// @return the current a value
var b2Sweep_GetA(b2Sweep *sweep);

#endif // __B2_SWEEP_H__
