#ifndef __B2_CONTACT_LISTENER_H__
#define __B2_CONTACT_LISTENER_H__

/// \brief Create contactListener.
/// Callback signatures :
/// - void beginContactCallback(b2Contact *contact, void *);
/// - void endContactCallback(b2Contact *contact, void *);
/// - void preSolveCallback(b2Contact *contact, const b2Manifold* oldManifold, void *);
/// - void postSolveCallback(b2Contact *contact, const b2ContactImpulse* impulse, void *);
b2ContactListener *b2ContactListener_Create(void *beginContactCallback, void *endContactCallback,
	void *preSolveCallback, void *postSolveCallback);

/// Destroy b2ContactListener instance
void b2ContactListener_Destroy(b2ContactListener *listener);

/// Set userData
/// user data is passed to the callback functions
void b2ContactListener_SetUserData(b2ContactListener *listener, void *userData);

/// Get userData
/// user data is passed to the callback functions
void* b2ContactListener_GetUserData(b2ContactListener *listener);

/// Set BeginContact Callback
/// Callback functions are called by the physics engine
void b2ContactListener_SetBeginContactCallback(b2ContactListener *listener, void *callback);

/// Get BeginContact Callback
/// Callback functions are called by the physics engine
void* b2ContactListener_GetBeginContactCallback(b2ContactListener *listener);

/// Set EndContact Callback
/// Callback functions are called by the physics engine
void b2ContactListener_SetEndContactCallback(b2ContactListener *listener, void *callback);

/// Get EndContact Callback
/// Callback functions are called by the physics engine
void* b2ContactListener_GetEndContactCallback(b2ContactListener *listener);

/// Set Presolve Callback
/// Callback functions are called by the physics engine
void b2ContactListener_SetPresolveCallback(b2ContactListener *listener, void *callback);

/// Get Presolve Callback
/// Callback functions are called by the physics engine
void* b2ContactListener_GetPresolveCallback(b2ContactListener *listener);

/// Set PostSolve Callback
/// Callback functions are called by the physics engine
void b2ContactListener_SetPostSolveCallback(b2ContactListener *listener, void *callback);

/// Get Postsolve Callback
/// Callback functions are called by the physics engine
void* b2ContactListener_GetPostSolveCallback(b2ContactListener *listener);


/// Set Callbacks
/// Callback functions are called by the physics engine
void b2ContactListener_SetCallbacks(b2ContactListener *listener, void *beginContactCallback, void *endContactCallback,
	void *preSolveCallback, void *postSolveCallback);

#endif // __B2_CONTACT_LISTENER_H__
