#ifndef __B2_DEBUG_DRAW_H__
#define __B2_DEBUG_DRAW_H__

/// Create b2DebugDraw.
/// Functions can be implemented.
/// Callback signatures :
/// - void drawPolygonFunc(const VECTOR *vertices, var vertexCount, const COLOR *color, void *userData);
/// - void drawSolidPolygonFunc(const VECTOR* vertices, var vertexCount, const COLOR *color, void *userData);
/// - void drawCircleFunc(const VECTOR *center, var radius, const COLOR *color, void *userData);
/// - void drawSolidCircleFunc(const VECTOR *center, var radius, const VECTOR *axis, const COLOR *color, void *userData);
/// - void drawSegmentFunc(const VECTOR *p1, const VECTOR *p2, const COLOR *color, void *userData);
/// - void drawTransformFunc(const b2Transform *xf, void *userData);
b2DebugDraw* b2DebugDraw_Create(void *drawPolygonFunc, void *drawSolidPolygonFunc, void *drawCircleFunc,
			void *drawSolidCircleFunc, void *drawSegmentFunc, void *drawTransformFunc);

/// Destroy b2DebugDraw
void b2DebugDraw_Destroy(b2DebugDraw *draw);

/// Set the drawing flags.
void b2DebugDraw_SetFlags(b2DebugDraw *draw, var flags);

/// Get the drawing flags.
var b2DebugDraw_GetFlags(b2DebugDraw *draw);
	
/// Append flags to the current flags.
void b2DebugDraw_AppendFlags(b2DebugDraw *draw, var flags);

/// Set userData
/// user data is passed to the callback functions
void b2DebugDraw_SetUserData(b2DebugDraw *draw, void *userData);

/// Get userData
/// user data is passed to the callback functions
void* b2DebugDraw_GetUserData(b2DebugDraw *draw);

/// Set DrawPolygonFunc
/// Functions are called by  the engine when set
void b2DebugDraw_SetDrawPolygonFunc(b2DebugDraw *draw, void *drawPolygonFunc);

/// Get DrawPolygon function
/// Functions are called by  the engine when set
void* b2DebugDraw_GetDrawPolygonFunc(b2DebugDraw *draw);

/// Set DrawSolidPolgyonFunc
/// Functions are called by  the engine when set
void b2DebugDraw_SetDrawSolidPolygonFunc(b2DebugDraw *draw, void *drawSolidPolygonFunc);

/// Get DrawPolygon function
/// Functions are called by  the engine when set
void* b2DebugDraw_GetDrawSolidPolygonFunc(b2DebugDraw *draw);

/// Set DrawCircleFunc
/// Functions are called by  the engine when set
void b2DebugDraw_SetDrawCircleFunc(b2DebugDraw *draw, void *drawCircleFunc);

/// Get DrawCircleFunc
/// Functions are called by  the engine when set
void* b2DebugDraw_GetDrawCircleFunc(b2DebugDraw *draw);

/// Set DrawSolidCircleFunc
/// Functions are called by  the engine when set
void b2DebugDraw_SetDrawSolidCircleFunc(b2DebugDraw *draw, void *drawSolidCircleFunc);

/// Get DrawSolidCircleFunc
/// Functions are called by  the engine when set
void* b2DebugDraw_GetDrawSolidCircleFunc(b2DebugDraw *draw);

/// Set DrawSegmentFunc
/// Functions are called by  the engine when set
void b2DebugDraw_SetDrawSegmentFunc(b2DebugDraw *draw, void *drawSegmentFunc);

/// Get DrawSegmentFunc
/// Functions are called by  the engine when set
void* b2DebugDraw_GetDrawSegmentFunc(b2DebugDraw *draw);

/// Set DrawTransformFunc
/// Functions are called by  the engine when set
void b2DebugDraw_SetDrawTransformFunc(b2DebugDraw *draw, void *drawTransformFunc);

/// Get DrawTransformFunc
/// Functions are called by  the engine when set
void* b2DebugDraw_GetDrawTransformFunc(b2DebugDraw *draw);

/// Set Callbacks
/// Functions are called by  the engine when set
void b2DebugDraw_SetCallback(b2DebugDraw *draw, void *drawPolygonFunc, void *drawSolidPolygonFunc, void *drawCircleFunc,
			void *drawSolidCircleFunc, void *drawSegmentFunc, void *drawTransformFunc);

#endif // __B2_DEBUG_DRAW_H__
