#ifndef __B2_DESTRUCTION_LISTENER_H__
#define __B2_DESTRUCTION_LISTENER_H__

/// Create b2DestructionListener.
/// Functions can be implemented.
/// Callback signatures : 
/// - void jointCallback(b2Joint *joint, void *userData);
/// - void fixtureCallback(b2Fixture *joint, void *userData);
b2DestructionListener* b2DestructionListener_Create(void *jointCallback, void *fixtureCallback);

/// Destroy b2DestructionListener
void b2DestructionListener_Destroy(b2DestructionListener *listener);

/// Set userData
/// user data is passed to the callback functions
void b2DestructionListener_SetUserData(b2DestructionListener *listener, void *data);

/// Get userData
/// user data is passed to the callback functions
void* b2DestructionListener_GetUserData(b2DestructionListener *listener);

/// Set Joint Callback
/// Callback functions are called by the physics engine
void b2DestructionListener_SetJointCallback(b2DestructionListener *listener, void *callback);

/// Get Joint Callback
/// Callback functions are called by the physics engine
void* b2DestructionListener_GetJointCallback(b2DestructionListener *listener);

/// Set Fixture Callback
/// Callback functions are called by the physics engine
void b2DestructionListener_SetFixtureCallback(b2DestructionListener *listener, void *callback);

/// Get Fixture Callback
/// Callback functions are called by the physics engine
void* b2DestructionListener_GetFixtureCallback(b2DestructionListener *listener);

/// Set Callbacks
/// Callback functions are called by the physics engine
void b2DestructionListener_SetCallbacks(b2DestructionListener *listener, void *jointCallback, void *fixtureCallback);


#endif // __B2_DESTRUCTION_LISTENER_H__
