#ifndef __B2_DISTANCE_JOINT_DEF_H__
#define __B2_DISTANCE_JOINT_DEF_H__

/// Create b2DistanceJointDef
b2DistanceJointDef* b2DistanceJointDef_Create();

/// Destroy b2DistanceJointDef
/// @param def the joint definition.
void b2DistanceJointDef_Destroy(b2DistanceJointDef *def);

/// Initialize the bodies, anchors, and length using the world
/// anchors.
void b2DistanceJointDef_Initialize(b2DistanceJointDef *def, b2Body* bodyA, b2Body* bodyB,
				const VECTOR *anchorA, const VECTOR *anchorB);

/// The local anchor point relative to body1's origin.
void b2DistanceJointDef_SetLocalAnchorA(b2DistanceJointDef *def, VECTOR *localAnchor);

/// The local anchor point relative to body1's origin.
void b2DistanceJointDef_GetLocalAnchorA(b2DistanceJointDef *def, VECTOR *outLocalAnchor);

/// The local anchor point relative to body1's origin.
void b2DistanceJointDef_SetLocalAnchorB(b2DistanceJointDef *def, VECTOR *localAnchor);

/// The local anchor point relative to body2's origin.
void b2DistanceJointDef_GetLocalAnchorB(b2DistanceJointDef *def, VECTOR *outLocalAnchor);

/// The natural length between the anchor points.
void b2DistanceJointDef_SetLength(b2DistanceJointDef *def, var length);

/// The natural length between the anchor points.
var b2DistanceJointDef_GetLength(b2DistanceJointDef *def);

/// The mass-spring-damper frequency in Hertz.
void b2DistanceJointDef_SetFrequencyHz(b2DistanceJointDef *def, var frequencyHz);

/// The mass-spring-damper frequency in Hertz.
var b2DistanceJointDef_GetFrequencyHz(b2DistanceJointDef *def);

/// The damping ratio. 0 = no damping, 1 = critical damping.
void b2DistanceJointDef_SetDampingRatio(b2DistanceJointDef *def, var dampingRatio);

/// The damping ratio. 0 = no damping, 1 = critical damping.
var b2DistanceJointDef_GetDampingRatio(b2DistanceJointDef *def);

#endif // __B2_DISTANCE_JOINT_DEF_H__
