#ifndef __B2_GEAR_JOINT_DEF_H__
#define __B2_GEAR_JOINT_DEF_H__

/// Create b2GearJoint
b2GearJointDef* b2GearJointDef_Create();

/// Destroy b2GearJoint
/// @param def the joint definition.
void b2GearJointDef_Destroy(b2GearJointDef *def);

/// Set the first revolute/prismatic joint attached to the gear joint.
/// @param def the joint definition
/// @param joint1 the first joint
void b2GearJointDef_SetJoint1(b2GearJointDef *def, b2Joint *joint1);

/// Get the first revolute/prismatic joint attached to the gear joint.
/// @param def the joint definition
b2Joint* b2GearJointDef_GetJoint1(b2GearJointDef *def);

/// Set the second revolute/prismatic joint attached to the gear joint.
/// @param def the joint definition
/// @param joint1 the first joint
void b2GearJointDef_SetJoint2(b2GearJointDef *def, b2Joint *joint2);

/// Get the second revolute/prismatic joint attached to the gear joint.
/// @param def the joint definition
b2Joint* b2GearJointDef_GetJoint1(b2GearJointDef *def);


/// Set the gear ratio.
/// @param def the joint definition
/// @param joint1 the first joint
void b2GearJointDef_SetRatio(b2GearJointDef *def, var ratio);

/// Get the second revolute/prismatic joint attached to the gear joint.
/// @param def the joint definition
var b2GearJointDef_GetRatio(b2GearJointDef *def);

/// \brief Set Body A
///
/// @param def The b2JointDef
/// @param body The body
#define b2GearJointDef_SetBodyA(def, body) b2JointDef_SetBodyA(def, body)

/// \brief Set Body B
///
/// @param def The b2JointDef
/// @param body The body
#define b2GearJointDef_SetBodyB(def, body) b2JointDef_SetBodyB(def, body)

#endif // __B2_GEAR_JOINT_DEF_H__
