#ifndef __B2_JOINT_H__
#define __B2_JOINT_H__

/// Get the type of the concrete joint.
var GetType(b2Joint *joint);

/// Get the first body attached to this joint.
b2Body* b2Joint_GetBodyA(b2Joint *joint);

/// Get the second body attached to this joint.
b2Body* b2Joint_GetBodyB(b2Joint *joint);

/// Get the anchor point on bodyA in world coordinates.
void b2Joint_GetAnchorA(b2Joint *joint, VECTOR *outAnchorA);

/// Get the anchor point on bodyB in world coordinates.
void b2Joint_GetAnchorB(b2Joint *joint, VECTOR *outAnchorB);

/// Get the reaction force on body2 at the joint anchor in Newtons.
void b2Joint_GetReactionForce(b2Joint *joint, var inv_dt, VECTOR *outReactionForce);

/// Get the reaction torque on body2 in N*m.
var b2Joint_GetReactionTorque(b2Joint *joint, var inv_dt);

/// Get the next joint the world joint list.
b2Joint* b2Joint_GetNext(b2Joint *joint);

/// Get the user data pointer.
void* b2Joint_GetUserData(b2Joint *joint);

/// Set the user data pointer.
void b2Joint_SetUserData(b2Joint *joint, void* data);

/// Short-cut function to determine if either body is inactive.
var b2Joint_IsActive(b2Joint *joint);

#endif __B2_JOINT_H__
