#ifndef __B2_JOINT_DEF_H__
#define __B2_JOINT_DEF_H__

/// Create b2JointDef
b2JointDef* b2JointDef_Create();

/// Destroy JointDef
/// @param def the joint definition.
void b2JointDef_Destroy(b2JointDef *def);

/// Set Joint type
/// The joint type is set automatically for concrete joint types.
/// @param def the joint definition.
/// @param type the joint type
void b2JointDef_SetType(b2JointDef *def, var type);

/// Get Joint type
/// The joint type is set automatically for concrete joint types.
/// @param def the joint definition.
var b2JointDef_GetType(b2JointDef *def);

/// Use this to attach application specific data to your joints.
/// @param def the joint definition.
/// @param userData the data attached to the joint
void b2JointDef_SetUserData(b2JointDef *def, void *userData);

/// Get attached joint data
/// @param def the joint definition.
void* b2JointDef_GetUserData(b2JointDef *def);

/// The first attached body.
/// @param def the joint definition.
/// @param body the attached body.
void b2JointDef_SetBodyA(b2JointDef *def, b2Body *body);

/// The first attached body.
/// @param def the joint definition.
/// @param body the attached body.
b2Body* b2JointDef_GetBodyA(b2JointDef *def);

/// The second attached body.
/// @param def the joint definition.
/// @param body the attached body.
void b2JointDef_SetBodyB(b2JointDef *def, b2Body *body);

/// The second attached body.
/// @param def the joint definition.
/// @param body the attached body.
b2Body* b2JointDef_GetBodyB(b2JointDef *def);

/// Set this flag to true if the attached bodies should collide.
/// @param def the joint definition.
/// @param flag the flag.
void b2JointDef_SetCollideConnected(b2JointDef *def, var flag);

/// Set this flag to true if the attached bodies should collide.
/// @param def the joint definition.
var b2JointDef_GetCollideConnected(b2JointDef *def);

#endif // __B2_JOINT_DEF_H__