#ifndef __B2_LINE_JOINT_DEF_H__
#define __B2_LINE_JOINT_DEF_H__

/// Create b2LineJointDef
b2LineJointDef* b2LineJointDef_Create();

/// Destroy b2LineJointDef
/// @param def the joint definition.
void b2LineJointDef_Destroy(b2LineJointDef *def);

// Initialize the bodies, anchors, axis, and reference angle using the world
/// anchor and world axis.
void b2LineJointDef_Initialize(b2LineJointDef *def, b2Body* bodyA, b2Body* bodyB, const VECTOR *anchor, const VECTOR *axis);

/// The local anchor point relative to body1's origin.
void b2LineJointDef_SetLocalAnchorA(b2LineJointDef *def, VECTOR *localAnchor);

/// The local anchor point relative to body1's origin.
void b2LineJointDef_GetLocalAnchorA(b2LineJointDef *def, VECTOR *outLocalAnchor);

/// The local anchor point relative to body2's origin.
void b2LineJointDef_SetLocalAnchorB(b2LineJointDef *def, VECTOR *localAnchor);

/// The local anchor point relative to body2's origin.
void b2LineJointDef_GetLocalAnchorB(b2LineJointDef *def, VECTOR *outLocalAnchor);

/// The local translation axis in body1.
void b2LineJointDef_SetLocalAnchorB(b2LineJointDef *def, VECTOR *localAnchor);

/// The local translation axis in body1.
void b2LineJointDef_GetLocalAnchorB(b2LineJointDef *def, VECTOR *outLocalAxis);

/// Enable the joint limit.
void b2LineJointDef_SetEnableLimit(b2LineJointDef *def, var enableLimit);

/// Disable the joint limit.
var b2LineJointDef_GetEnableLimit(b2LineJointDef *def);

/// The lower translation limit, usually in meters.
void b2LineJointDef_SetLowerTranslation(b2LineJointDef *def, var lowerTranslation);

/// The lower translation limit, usually in meters.
var b2LineJointDef_GetLowerTranslation(b2LineJointDef *def);

/// The lower translation limit, usually in meters.
void b2LineJointDef_SetUpperTranslation(b2LineJointDef *def, var upperTranslation);

/// The lower translation limit, usually in meters.
var b2LineJointDef_GetUpperTranslation(b2LineJointDef *def);

/// Enable the joint limit.
void b2LineJointDef_SetEnableMotor(b2LineJointDef *def, var enableMotor);

/// Disable the joint limit.
var b2LineJointDef_GetEnableMotor(b2LineJointDef *def);

/// The maximum motor torque, usually in N-m.
void b2LineJointDef_SetMaxMotorForce(b2LineJointDef *def, var maxMotorForce);

/// The maximum motor torque, usually in N-m.
var b2LineJointDef_GetMaxMotorForce(b2LineJointDef *def);

/// The maximum motor torque, usually in N-m.
void b2LineJointDef_MotorSpeed(b2LineJointDef *def, var motorSpeed);

/// The maximum motor torque, usually in N-m.
var b2LineJointDef_GetMotorSpeed(b2LineJointDef *def);

#endif // __B2_LINE_JOINT_DEF_H__
