#ifndef __B2_PRISMATIC_JOINT_DEF_H__
#define __B2_PRISMATIC_JOINT_DEF_H__


/// Create b2PrismaticJointdef
b2PrismaticJointDef *b2PrismaticJointDef_Create();

/// Delete b2PrismaticJointDef
void b2PrismaticJointDef_Destroy(b2PrismaticJointDef *def);

/// Initialize the bodies, anchors, axis, and reference angle using the world
/// anchor and world axis.
void b2PrismaticJointDef_Initialize(b2PrismaticJointDef *def, b2Body* bodyA, b2Body* bodyB, const VECTOR *anchor, const VECTOR *axis);

/// Set local anchor A
/// The local anchor point relative to body1's origin.
void b2PrismaticJointDef_SetLocalAnchorA(b2PrismaticJointDef *def, const VECTOR *localAnchor);

/// Get local anchor A
/// The local anchor point relative to body1's origin.
void b2PrismaticJointDef_GetLocalAnchorA(b2PrismaticJointDef *def, VECTOR *outLocalAnchor);

/// Set local anchor B
/// The local anchor point relative to body2's origin.
void b2PrismaticJointDef_SetLocalAnchorB(b2PrismaticJointDef *def, const VECTOR *localAnchor);

/// Get local anchor B
/// The local anchor point relative to body2's origin.
void b2PrismaticJointDef_GetLocalAnchorB(b2PrismaticJointDef *def, VECTOR *outLocalAnchor);

/// Set localAxis1
/// The local translation axis in body1.
void b2PrismaticJointDef_SetLocalAxis1(b2PrismaticJointDef *def, const VECTOR *localAxis);

/// Get localAxis1
/// The local translation axis in body1.
void b2PrismaticJointDef_GetLocalAxis1(b2PrismaticJointDef *def, VECTOR *outLocalAxis);

/// Set Reference angle
/// The constrained angle between the bodies: body2_angle - body1_angle.
void b2PrismaticJointDef_SetReferenceAngle(b2PrismaticJointDef *def, var referenceAngle);

/// Get Reference angle
/// The constrained angle between the bodies: body2_angle - body1_angle.
var b2PrismaticJointDef_GetReferenceAngle(b2PrismaticJointDef *def);

/// Enable/disable the joint limit.
void b2PrismaticJointDef_SetEnableLimit(b2PrismaticJointDef *def, var enableLimit);

/// Enable/disable the joint limit.
var b2PrismaticJointDef_GetEnableLimit(b2PrismaticJointDef *def);

/// The lower translation limit, usually in meters.
void b2PrismaticJointDef_SetLowerTranslation(b2PrismaticJointDef *def, var lowerTranslation);

/// The lower translation limit, usually in meters.
var b2PrismaticJointDef_GetLowerTranslation(b2PrismaticJointDef *def);

/// The upper translation limit, usually in meters.
void b2PrismaticJointDef_SetUpperTranslation(b2PrismaticJointDef *def, var upperTranslation);

/// The upper translation limit, usually in meters.
var b2PrismaticJointDef_GetUpperTranslation(b2PrismaticJointDef *def);

/// Enable/disable the joint motor.
void b2PrismaticJointDef_SetEnableMotor(b2PrismaticJointDef *def, var enableMotor);

/// Enable/disable the joint motor.
var b2PrismaticJointDef_GetEnableMotor(b2PrismaticJointDef *def);

/// The maximum motor torque, usually in N-m.
void b2PrismaticJointDef_SetMaxMotorForce(b2PrismaticJointDef *def, var maxMotorForce);

/// The maximum motor torque, usually in N-m.
var b2PrismaticJointDef_GetMaxMotorForce(b2PrismaticJointDef *def);

/// The desired motor speed in radians per second.
void b2PrismaticJointDef_SetMotorSpeed(b2PrismaticJointDef *def, var motorSpeed);

/// The desired motor speed in radians per second.
var b2PrismaticJointDef_GetMotorSpeed(b2PrismaticJointDef *def);

#endif // __B2_PRISMATIC_JOINT_DEF_H__
