#ifndef __B2_PULLEY_JOINT_DEF_H__
#define __B2_PULLEY_JOINT_DEF_H__

/// Create b2PulleyJoint
b2PulleyJointDef *b2PulleyJointDef_Create();

/// Destroy b2PulleyJoint
void b2PulleyJointDef_Destroy(b2PulleyJointDef *def);

/// Initialize the bodies, anchors, lengths, max lengths, and ratio using the world anchors.
void b2PulleyJointDef_Initialize(b2PulleyJointDef *def, b2Body* bodyA, b2Body* bodyB,
				const VECTOR *groundAnchorA, const VECTOR *groundAnchorB,
				const VECTOR *anchorA, const VECTOR *anchorB,
				var ratio);

/// The first ground anchor in world coordinates. This point never moves.
void b2PulleyJointDef_SetGroundAnchorA(b2PulleyJointDef *def, const VECTOR *groundAnchorA);

/// The first ground anchor in world coordinates. This point never moves.
void b2PulleyJointDef_GetGroundAnchorA(b2PulleyJointDef *def, VECTOR *outGroundAnchorA);

/// The second ground anchor in world coordinates. This point never moves.
void b2PulleyJointDef_SetGroundAnchorB(b2PulleyJointDef *def, const VECTOR *groundAnchorB);

/// The second ground anchor in world coordinates. This point never moves.
void b2PulleyJointDef_GetGroundAnchorB(b2PulleyJointDef *def, VECTOR *outGroundAnchorB);

/// The local anchor point relative to bodyA's origin.
void b2PulleyJointDef_SetLocalAnchorA(b2PulleyJointDef *def, const VECTOR *localAnchorA);

/// The local anchor point relative to bodyA's origin.
void b2PulleyJointDef_GetlocalAnchorA(b2PulleyJointDef *def, VECTOR *outLocalAnchorA);

/// The local anchor point relative to bodyB's origin.
void b2PulleyJointDef_SetLocalAnchorB(b2PulleyJointDef *def, const VECTOR *localAnchorB);

/// The local anchor point relative to bodyB's origin.
void b2PulleyJointDef_GetlocalAnchorB(b2PulleyJointDef *def, VECTOR *outLocalAnchorB);

/// The a reference length for the segment attached to bodyA.
void b2PulleyJointDef_SetLengthA(b2PulleyJointDef *def, var lengthA);

/// The a reference length for the segment attached to bodyA.
var b2PulleyJointDef_GetLengthA(b2PulleyJointDef *def);

/// The maximum length of the segment attached to bodyA.
void b2PulleyJointDef_SetMaxLengthA(b2PulleyJointDef *def, var maxLengthA);

/// The maximum length of the segment attached to bodyA.
var b2PulleyJointDef_GetMaxLengthA(b2PulleyJointDef *def);

/// The a reference length for the segment attached to bodyB.
void b2PulleyJointDef_SetLengthB(b2PulleyJointDef *def, var lengthB);

/// The a reference length for the segment attached to bodyB.
var b2PulleyJointDef_GetLengthB(b2PulleyJointDef *def);

/// The maximum length of the segment attached to bodyB.
void b2PulleyJointDef_SetMaxLengthB(b2PulleyJointDef *def, var maxLengthB);

/// The maximum length of the segment attached to bodyB.
var b2PulleyJointDef_GeMaxLengthB(b2PulleyJointDef *def);

/// The pulley ratio, used to simulate a block-and-tackle.
void b2PulleyJointDef_SetRatio(b2PulleyJointDef *def, var ratio);

/// The pulley ratio, used to simulate a block-and-tackle.
var b2PulleyJointDef_GetRatio(b2PulleyJointDef *def);

#endif // __B2_PULLEY_JOINT_DEF_H__
