#ifndef __B2_REVOLUTE_JOINT_DEF_H__
#define __B2_REVOLUTE_JOINT_DEF_H__

/// Create b2Reveolutejoint
b2RevoluteJointDef *b2RevoluteJointDef_Create();

/// Delete b2RevoluteJoint
void b2RevoluteJointDef_Destroy(b2RevoluteJointDef *def);

/// Initialize the bodies, anchors, and reference angle using a world
/// anchor point.
void b2RevoluteJointDef_Initialize(b2RevoluteJointDef *def, b2Body* bodyA, b2Body* bodyB, const VECTOR *anchor);

/// The local anchor point relative to bodyA's origin.
void b2RevoluteJointDef_SetLocalAnchorA(b2RevoluteJointDef *def, const VECTOR *localAnchorA);

/// The local anchor point relative to bodyA's origin.
void b2RevoluteJointDef_GetlocalAnchorA(b2RevoluteJointDef *def, VECTOR *outLocalAnchorA);

/// The local anchor point relative to bodyB's origin.
void b2RevoluteJointDef_SetLocalAnchorB(b2RevoluteJointDef *def, const VECTOR *localAnchorB);

/// The local anchor point relative to bodyB's origin.
void b2RevoluteJointDef_GetlocalAnchorB(b2RevoluteJointDef *def, VECTOR *outLocalAnchorB);

/// Set Reference angle
/// The body2 angle minus body1 angle in the reference state (radians).
void b2RevoluteJointDef_SetReferenceAngle(b2RevoluteJointDef *def, var referenceAngle);

/// Get Reference angle
/// The body2 angle minus body1 angle in the reference state (radians).
var b2RevoluteJointDef_GetReferenceAngle(b2RevoluteJointDef *def);

/// A flag to enable joint limits.
void b2RevoluteJointDef_SetEnableLimit(b2RevoluteJointDef *def, var enableLimit);

/// A flag to enable joint limits.
var b2RevoluteJointDef_GetEnableLimit(b2RevoluteJointDef *def);

/// The lower angle for the joint limit (radians).
void b2RevoluteJointDef_SetLowerAngle(b2RevoluteJointDef *def, var lowerAngle);

/// The lower angle for the joint limit (radians).
var b2RevoluteJointDef_GetLowerAngle(b2RevoluteJointDef *def);

/// The upper angle for the joint limit (radians).
void b2RevoluteJointDef_SetUpperAngle(b2RevoluteJointDef *def, var upperAngle);

/// The upper angle for the joint limit (radians).
var b2RevoluteJointDef_GetUpperAngle(b2RevoluteJointDef *def);

/// Enable/disable the joint motor.
void b2RevoluteJointDef_SetEnableMotor(b2RevoluteJointDef *def, var enableMotor);

/// Enable/disable the joint motor.
var b2RevoluteJointDef_GetEnableMotor(b2RevoluteJointDef *def);

/// The desired motor speed. Usually in radians per second.
void b2RevoluteJointDef_SetMotorSpeed(b2RevoluteJointDef *def, var motorSpeed);

/// The desired motor speed. Usually in radians per second.
var b2RevoluteJointDef_GetMotorSpeed(b2RevoluteJointDef *def);

/// The maximum motor torque used to achieve the desired motor speed.
/// Usually in N-m.
void b2RevoluteJointDef_SetMaxMotorTorque(b2RevoluteJointDef *def, var maxMotorTorque);

/// The maximum motor torque used to achieve the desired motor speed.
/// Usually in N-m.
var b2RevoluteJointDef_GetMaxMotorTorque(b2RevoluteJointDef *def);

#endif // __B2_REVOLUTE_JOINT_DEF_H__
