#include <acknex.h>
#include <default.c>

#define PI 3.14159265

// Include tests
#define PRAGMA_PATH "testbed";
#define PRAGMA_PATH "..\\include\\Box2D";

#include "ackb2d.h"

// Create a 2d vector
#define vec2(a, b) vector(a, b, 0)
// Tests count
#define TEST_COUNT 10

#include "test.h"
#include "test_ragdoll.h"
#include "test_compound.h"
#include "test_crank.h"
#include "test_gears.h"
#include "test_pulley.h"
#include "test_toi.h"
#include "test_polycollision.h"
#include "test_bridge.h"
#include "test_stacked_box.h"
#include "test_stacked_circle.h"
#include "test_body_types.h"
#include "test_theo_jansen.h"

STRING *tests[TEST_COUNT];
var current_test_id = 0;
var test_count = TEST_COUNT;
STRING *current_test_name = ""; 

// Last key states
var testbed_last_r = 0;
var testbed_last_cul = 0;
var testbed_last_cur = 0;

//////////////////////////////
// Function prototypes
//////////////////////////////
void init();
void clean();
void update();
void exit_current_test();

// Current test functions
void test_init();
void test_update();
void test_exit();

// Display current test number
var current_test_nb = 0;
PANEL *pan_current_test = 
{
	digits(0, 10, "%5.0f/", "*", 1, current_test_nb);
	digits(20, 10, "%5.0f", "*", 1, test_count);
	digits(70, 10, "%s", "*", 1, current_test_name);
	
	pos_x = -10;
	
	flags = SHOW;
}

// Display title
FONT *fnt_title = "Arial#16b";
TEXT *txt_title = 
{
	string(
				"3DGS A7/A8 Box2D Wrapper (Box2D 2.1.2)",
				"http://www.box2d.org/",
				"'Left'/'Right' arrows to go to previous/next example.",
				"'R' to reset current test.",
				"Mouse to drag a dynamic body",
			);
	font = fnt_title;
	flags = SHOW | ARIGHT;
	pos_x = 630;
	pos_y = 10;
}

FONT *fnt_test_name = "Arial#22b";
TEXT *txt_test_name = 
{
	string(str_test_name);
	font = fnt_test_name;
	
	blue = 255;
	green = 255;
	red = 0; 
	
	flags = SHOW | ARIGHT;
	
	pos_x = 630;
	pos_y = 100;
}

///
/// Main function
///
void main()
{
	//video_mode = 6;
	//video_switch(10, 0, 6);
	video_set(640, 360, NULL, NULL);
	vec_set(screen_color, vector(0x20, 0x20, 0x20));
	wait(1);
	
	
	// fps max
	fps_max = 60;	
	
	// Init testbed
	init();
	on_exit = clean;	
	
	// Main loop
	while(1)
	{
		update();
		wait(1);
	}
}

///
/// Init test array
///
void init()
{
	tests[0] = str_create("ragdoll");
	tests[1] = str_create("compound");
	tests[2] = str_create("crank");
	tests[3] = str_create("gears");
	tests[4] = str_create("pulley");
	tests[5] = str_create("bridge");
	tests[6] = str_create("stacked_box");
	tests[7] = str_create("stacked_circle");
	tests[8] = str_create("body_types");
	tests[9] = str_create("theojansen");
	/*tests[5] = str_create("toi");
	tests[6] = str_create("poly_collision");*/
	
	// Current test number
	current_test_id = 0;
	
	// Current test name
	str_cpy(current_test_name, "");
}

/// \brief Update testbed
///
/// Handle keys
// Update current test
void update()
{
	// Next test
	if ((key_cur == 1) && (testbed_last_cur == 0))
	{
		current_test_id ++;
		str_cpy(current_test_name, "");
		// Exit current test
		exit_current_test();
	}
	
	// Previous test
	if ((key_cul == 1) && (testbed_last_cul == 0))
	{
		current_test_id --;
		str_cpy(current_test_name, "");
		// Exit current test
		exit_current_test();
	}
	
	// Reset
	if ((key_r == 1) && (testbed_last_r == 0))
	{
		str_cpy(current_test_name, "");
		// Exit current test
		exit_current_test();
	}
	
	// Store key status
	testbed_last_cul = key_cul;
	testbed_last_cur = key_cur;
	testbed_last_r = key_r;
	
	// Wrap current test id in 0..test_count
	current_test_id = cycle(current_test_id, 0, TEST_COUNT);
	
	// Set current test id
	current_test_nb = current_test_id + 1;
	
	// Create new test if null
	if (str_cmp(current_test_name, ""))
	{
		if (tests[current_test_id] == NULL)
		{
			return;
		}
		str_cpy(current_test_name, tests[current_test_id]);
		
		// Init current test functions
		STRING *func_init = str_create(_chr(current_test_name));
		str_cat(func_init, "_init");
		
		STRING *func_update = str_create(_chr(current_test_name));
		str_cat(func_update, "_update");
		
		STRING *func_exit = str_create(_chr(current_test_name));
		str_cat(func_exit, "_exit");
		
		test_init = engine_getscript(_chr(func_init));
		test_update = engine_getscript(_chr(func_update));
		test_exit = engine_getscript(_chr(func_exit));
		
		if (test_init  != NULL)
		{
			test_init();
		}
		
		// Free pointers
		str_remove(func_init);
		str_remove(func_update);
		str_remove(func_exit);
	}
	
	// Update current test
	if (test_update != NULL)
	{
		test_update();	
	}
}

///
/// Clean testbed
///
void clean()
{
	// Exit current test
	exit_current_test();
	
	// Clear test array
	int i = 0;
	for(i = 0; i<TEST_COUNT; i++)
	{
		if (tests[i] != NULL)
		{
			str_remove(tests[i]);
		}
	}
}

///
/// Exit current test
///
void exit_current_test()
{
	if (test_exit != NULL)
	{
		test_exit();
	}
}