#ifndef __TEST_BODY_TYPES_H__
#define __TEST_BODY_TYPES_H__

/// \brief default variables for body types test
DEF_VARIABLES bt_def_vars;

/// \brief Attachment Body
b2Body *bt_attachment_body = NULL;

/// \brief Platform body
b2Body *bt_platform_body = NULL; 
var bt_speed = 3.0;

// Instruction panel
FONT *fnt_bt_instruction = "Arial#16";
TEXT *bt_pan_instruction = 
{
	string(
				"Keys : (d) dynamic, (s) static, (k) kinematic"
			);	
	
	font = fnt_sb_instruction;
		
	blue = 0;
	green = 0;
	red = 255;
	
	pos_x = 10;
	pos_y = 20; 
}

// Last key states
var bt_last_d = 0;
var bt_last_s = 0;
var bt_last_k = 0;

/// \brief Init body types test
void body_types_init()
{
	deftest_init(&bt_def_vars);
	DEF_VARIABLES *vars = &bt_def_vars;
	
	// Set title
	str_cpy(str_test_name, "Body Types");
	
	// Ground body
	b2BodyDef *ground_def = b2BodyDef_Create();
	b2BodyDef_SetPosition(ground_def, vec2(screen_size.x / 2 / vars->m_physScale, screen_size.y / vars->m_physScale));
	b2Body *ground = b2World_CreateBody(vars->m_world, ground_def);
	// Destroy unused objects
	b2BodyDef_Destroy(ground_def);
	
	// Define attachment
	{
		b2BodyDef *body_def = b2BodyDef_Create();
		b2BodyDef_SetType(body_def, B2_DYNAMICBODY);
		b2BodyDef_SetPosition(body_def, vec2(screen_size.x / 2 / vars->m_physScale, (screen_size.y - 100) / vars->m_physScale));
		
		bt_attachment_body = b2World_CreateBody(vars->m_world, body_def);
		
		b2PolygonShape *attachment_shape = b2PolygonShape_Create();
		b2PolygonShape_SetAsBox1(attachment_shape, 10 / vars->m_physScale, 40 / vars->m_physScale);
		b2Body_CreateFixture2(bt_attachment_body, attachment_shape, 10.0);
		
		// Destroy unused objects
		b2BodyDef_Destroy(body_def);
		b2PolygonShape_Destroy(attachment_shape);
	}
	
	// Define platform
	{
		b2BodyDef *body_def = b2BodyDef_Create();
		b2BodyDef_SetType(body_def, B2_DYNAMICBODY);
		b2BodyDef_SetPosition(body_def, vec2((screen_size.x / 2 - 80) / vars->m_physScale, (screen_size.y - 140) / vars->m_physScale));
		
		bt_platform_body = b2World_CreateBody(vars->m_world, body_def);
		
		b2PolygonShape *platform_shape = b2PolygonShape_Create();
		b2PolygonShape_SetAsBox2(platform_shape, 10 / vars->m_physScale, 80 / vars->m_physScale, vec2(80.0 / vars->m_physScale, 0.0), 0.5 * PI);
		
		b2FixtureDef *platform_fixture = b2FixtureDef_Create();
		b2FixtureDef_SetShape(platform_fixture, platform_shape);
		b2FixtureDef_SetFriction(platform_fixture, 0.6);
		b2FixtureDef_SetDensity(platform_fixture, 2.0);
		b2Body_CreateFixture(bt_platform_body, platform_fixture);
		
		b2RevoluteJointDef *joint_def = b2RevoluteJointDef_Create();
		b2RevoluteJointDef_Initialize(joint_def, bt_attachment_body, bt_platform_body, vec2((screen_size.x / 2 / vars->m_physScale), (screen_size.y - 140) / vars->m_physScale));
		b2RevoluteJointDef_SetMaxMotorTorque(joint_def, 50.0);
		b2RevoluteJointDef_SetEnableMotor(joint_def, 1);
		b2World_CreateJoint(vars->m_world, joint_def);
		
		b2PrismaticJointDef *pjd = b2PrismaticJointDef_Create();
		b2PrismaticJointDef_Initialize(pjd, ground, bt_platform_body, vec2((screen_size.x / 2 / vars->m_physScale), (screen_size.y - 140) / vars->m_physScale), vec2(1.0, 0));
		b2PrismaticJointDef_SetMaxMotorForce(pjd, 1000.0);
		b2PrismaticJointDef_SetEnableMotor(pjd, 1);
		b2PrismaticJointDef_SetLowerTranslation(pjd, -5.0);
		b2PrismaticJointDef_SetUpperTranslation(pjd, 5.0);
		b2PrismaticJointDef_SetEnableLimit(pjd, 1);
		
		b2World_CreateJoint(vars->m_world, pjd);
		
		// Destroy unused objects
		b2BodyDef_Destroy(body_def);
		b2PolygonShape_Destroy(platform_shape);
		b2FixtureDef_Destroy(platform_fixture);
		b2RevoluteJointDef_Destroy(joint_def);
		b2PrismaticJointDef_Destroy(pjd);
	}
	
	// Define payload
	{
		b2BodyDef *body_def = b2BodyDef_Create();
		b2BodyDef_SetType(body_def, B2_DYNAMICBODY);
		b2BodyDef_SetPosition(body_def, vec2(screen_size.x / 2 / vars->m_physScale, (screen_size.y / 2 - 40) / vars->m_physScale));
		
		b2Body *body = b2World_CreateBody(vars->m_world, body_def);
		
		b2PolygonShape *shape = b2PolygonShape_Create();
		b2PolygonShape_SetAsBox1(shape, 20 / vars->m_physScale, 20 / vars->m_physScale);
		
		b2FixtureDef *fixture_def = b2FixtureDef_Create();
		b2FixtureDef_SetShape(fixture_def, shape);
		b2FixtureDef_SetFriction(fixture_def, 0.6);
		b2FixtureDef_SetFriction(fixture_def, 2.0);
		
		b2Body_CreateFixture(body, fixture_def);
		
		// Destroy unused objects
		b2BodyDef_Destroy(body_def);
		b2PolygonShape_Destroy(shape);
		b2FixtureDef_Destroy(fixture_def);
	}	
	
	// Show instruction panel
	bt_pan_instruction->flags |= SHOW;
}

/// \brief Update current test suite
void body_types_update()
{
	deftest_update(&bt_def_vars);
	DEF_VARIABLES *vars = &bt_def_vars;
	
	// key d
	if (key_d == 1 && bt_last_d == 0)
	{
		b2Body_SetType(bt_platform_body, B2_DYNAMICBODY);
	}
	
	// key s
	if (key_s == 1 && bt_last_s == 0)
	{
		b2Body_SetType(bt_platform_body, B2_STATICBODY);
	}
	
	// key k
	if (key_k == 1 && bt_last_k == 0)
	{
		b2Body_SetType(bt_platform_body, B2_KINEMATICBODY);
		b2Body_SetLinearVelocity(bt_platform_body, vec2(-bt_speed, 0.0));
		b2Body_SetAngularVelocity(bt_platform_body, 0.0);
	}
	
	// Save current key states
	bt_last_d = 0;
	bt_last_s = 0;
	bt_last_k = 0;
	
	// Drive kinematic body
	if (b2Body_GetType(bt_platform_body) == B2_KINEMATICBODY)
	{
		VECTOR p;
		VECTOR v;
		
		/*b2Transform *platform_transform = b2Transform_Create();
		
		b2Body_GetTransform(bt_platform_body, platform_transform);
		b2Transform_GetPosition(platform_transform, &p);
		b2Transform_Destroy(platform_transform);*/
		b2Body_GetPosition(bt_platform_body, &p);
		
		b2Body_GetLinearVelocity(bt_platform_body, &v);
		
		if ((p.x < (40 / vars->m_physScale) && v.x < 0.0) || (p.x > ((screen_size.x - 200) / vars->m_physScale)  && v.x > 0.0))
		{
			v.x = -v.x;
			b2Body_SetLinearVelocity(bt_platform_body, &v);
		}
	}
}

/// \brief Clean current test suite
void body_types_exit()
{
	deftest_exit(&bt_def_vars);
	
	// Show instruction panel
	bt_pan_instruction->flags &= ~SHOW;
}

#endif // __TEST_BODY_TYPES_H__
