#ifndef __TEST_BRIDGE_H__
#define __TEST_BRIDGE_H__

/// \brief default variables for bridge test
DEF_VARIABLES bridge_def_vars;

/// \brief Init bridge test
void bridge_init()
{
	deftest_init(&bridge_def_vars);
	DEF_VARIABLES *vars = &bridge_def_vars;
	
	// Set title
	str_cpy(str_test_name, "Bridge");
	
	b2BodyDef *ground_def = b2BodyDef_Create();
	b2BodyDef_SetPosition(ground_def, vec2(screen_size.x / 2 / vars->m_physScale, screen_size.y / vars->m_physScale));
	b2Body *ground = b2World_CreateBody(vars->m_world, ground_def);
	
	VECTOR anchor;
	int i;
	b2Body *body = NULL;
	
	// Bridge
	{
		b2PolygonShape *shape_def = b2PolygonShape_Create();
		b2FixtureDef *fixture_def = b2FixtureDef_Create();
		
		b2PolygonShape_SetAsBox1(shape_def, 24 / vars->m_physScale, 5 / vars->m_physScale);
		b2FixtureDef_SetShape(fixture_def, shape_def);
		b2FixtureDef_SetDensity(fixture_def, 20.0);
		b2FixtureDef_SetFriction(fixture_def, 0.2);
		
		b2BodyDef *body_def = b2BodyDef_Create();
		b2BodyDef_SetType(body_def, B2_DYNAMICBODY);
		
		b2RevoluteJointDef *joint_def = b2RevoluteJointDef_Create();
		int num_planks = 10;
		
		b2RevoluteJointDef_SetLowerAngle(joint_def, -15 / (180 / PI));
		b2RevoluteJointDef_SetUpperAngle(joint_def, 15 / (180 / PI));
		b2RevoluteJointDef_SetEnableLimit(joint_def, 1);
		
		b2Body *prev_body = ground;
		for (i = 0; i < num_planks; i++)
		{
			b2BodyDef_SetPosition(body_def, vec2((100 + 22 + 44 * i) / vars->m_physScale, 250 / vars->m_physScale));
			body = b2World_CreateBody(vars->m_world, body_def);
			b2Body_CreateFixture(body, fixture_def);
			
			vec_set(&anchor, vec2((100 + 44 * i) / vars->m_physScale, 250 / vars->m_physScale));
			b2RevoluteJointDef_Initialize(joint_def, prev_body, body, &anchor);
			b2World_CreateJoint(vars->m_world, joint_def);
			
			prev_body = body;
		}
		
		vec_set(&anchor, vec2((100 + 44 * num_planks) / vars->m_physScale, 250 / vars->m_physScale));
		b2RevoluteJointDef_Initialize(joint_def, prev_body, ground, &anchor);
		b2World_CreateJoint(vars->m_world, joint_def);
		
		// Destroy unused objects
		b2PolygonShape_Destroy(shape_def);
		b2FixtureDef_Destroy(fixture_def);
		b2BodyDef_Destroy(body_def);
		b2BodyDef_Destroy(ground_def);
		b2RevoluteJointDef_Destroy(joint_def);
	}
	
	// Spawn in a bunch of crap
	b2BodyDef *body_def = b2BodyDef_Create();
	b2BodyDef_SetType(body_def, B2_DYNAMICBODY);
	b2PolygonShape *box_shape = b2PolygonShape_Create();
	
	b2FixtureDef *fixture_def = b2FixtureDef_Create();
	b2FixtureDef_SetShape(fixture_def, box_shape);
	b2FixtureDef_SetDensity(fixture_def, 1.0);
	b2FixtureDef_SetFriction(fixture_def, 0.3);
	b2FixtureDef_SetRestitution(fixture_def, 0.1);
	
	for (i = 0; i < 10; i++)
	{
		b2PolygonShape_SetAsBox1(box_shape, (random(5) + 10) / vars->m_physScale, (random(5) + 10) / vars->m_physScale);
		b2BodyDef_SetPosition(body_def, vec2((random(400) + 120) / vars->m_physScale, (random(150) + 50) / vars->m_physScale));
		b2BodyDef_SetAngle(body_def, random(PI));
		
		body = b2World_CreateBody(vars->m_world, body_def);
		b2Body_CreateFixture(body, fixture_def);
	}
	
	// Destroy objects
	b2PolygonShape_Destroy(box_shape);
	
	//////////////////////////////
	// Circles
	//////////////////////////////
	b2CircleShape *circle_shape = b2CircleShape_Create();
	b2FixtureDef_SetShape(fixture_def, circle_shape);
	
	for (i = 0; i < 10; i++)
	{
		b2CircleShape_SetRadius(circle_shape, (random(5) + 10) / vars->m_physScale);
		b2BodyDef_SetPosition(body_def, vec2((random(400) + 120) / vars->m_physScale, (random(150) + 50) / vars->m_physScale));
		b2BodyDef_SetAngle(body_def, random(PI));
		
		body = b2World_CreateBody(vars->m_world, body_def);
		b2Body_CreateFixture(body, fixture_def);	
	}
	
	// Destroy objects
	b2CircleShape_Destroy(circle_shape);
	b2BodyDef_Destroy(body_def);
	b2FixtureDef_Destroy(fixture_def);
}

/// \brief Update current test suite
void bridge_update()
{
	deftest_update(&bridge_def_vars);
}

/// \brief Clean current test suite
void bridge_exit()
{
	deftest_exit(&bridge_def_vars);
}

#endif // __TEST_BRIDGE_H__
