#ifndef __TEST_COMPOUND_H__
#define __TEST_COMPOUND_H__

/// \brief default variables for cmpound test
DEF_VARIABLES cmp_def_vars;

/// \brief Init ragdoll test
void compound_init()
{
	deftest_init(&rag_def_vars);
	DEF_VARIABLES *vars = &rag_def_vars;
	
	// Set title
	str_cpy(str_test_name, "Compound Shapes");
	
	b2BodyDef *body_def = NULL;
	b2Body *body = NULL;
	int i = 0;
	var x;
	
	// Circles
	{
		b2CircleShape *cd1 = b2CircleShape_Create();	
		b2CircleShape_SetRadius(cd1, 15.0/vars->m_physScale);
		b2CircleShape_SetPosition(cd1, vec2(-15 / vars->m_physScale, 15.0 / vars->m_physScale));
		
		b2CircleShape *cd2 = b2CircleShape_Create();	
		b2CircleShape_SetRadius(cd2, 15.0/vars->m_physScale);
		b2CircleShape_SetPosition(cd2, vec2(15 / vars->m_physScale, 15.0 / vars->m_physScale));
		
		body_def = b2BodyDef_Create();
		b2BodyDef_SetType(body_def, B2_DYNAMICBODY);
		
		int i = 0;
		for (i = 0; i < 5; i++)
		{
			x = 320.0 + random(6.0) - 3;
			b2BodyDef_SetPosition(body_def, vec2((x + 150.0) / vars->m_physScale, (31.5 + 75.0 * -i + 300) / vars->m_physScale));
			b2BodyDef_SetAngle(body_def, random(PI * 2) - PI);
			body = b2World_CreateBody(vars->m_world, body_def);
			b2Body_CreateFixture2(body, cd1, 2.0);
			b2Body_CreateFixture2(body, cd2, 0.0);
		}
		
		// Destroy shapes
		b2CircleShape_Destroy(cd1);
		b2CircleShape_Destroy(cd2);
		// Destroy bodydef
		b2BodyDef_Destroy(body_def);
	}
	
	// Polygons
	{
		b2PolygonShape *pd1 = b2PolygonShape_Create();
		b2PolygonShape_SetAsBox1(pd1, 7.5 / vars->m_physScale, 15.0 / vars->m_physScale);
		
		b2PolygonShape *pd2 = b2PolygonShape_Create();
		b2PolygonShape_SetAsBox2(pd2, 7.5 / vars->m_physScale, 15.0 / vars->m_physScale, vec2(0.0, -15.0 / vars->m_physScale), 0.5 * PI);
		
		body_def = b2BodyDef_Create();
		b2BodyDef_SetType(body_def, B2_DYNAMICBODY);
		
		int i = 0;
		for (i = 0; i < 5; i++)
		{
			x = 320.0 + random(6.0) - 3;
			b2BodyDef_SetPosition(body_def, vec2((x - 150.0) / vars->m_physScale, (31.5 + 75.0 * -i + 300) / vars->m_physScale));
			b2BodyDef_SetAngle(body_def, random(PI * 2) - PI);
			body = b2World_CreateBody(vars->m_world, body_def);
			b2Body_CreateFixture2(body, pd1, 2.0);
			b2Body_CreateFixture2(body, pd2, 2.0);
		}
		
		// Destroy shapes
		b2PolygonShape_Destroy(pd1);
		b2PolygonShape_Destroy(pd2);
		// Destroy bodyDef
		b2BodyDef_Destroy(body_def);
	}
	
	// Triangles
	{
		b2Transform *xf1 = b2Transform_Create();
		b2Transform_SetAngle(xf1, 0.3524 * PI);
		// Rotation matrix
		VECTOR R[2];
		b2Transform_GetRotationMat(xf1, R);
		VECTOR tmp;
		mulmv(&tmp, R, vec2(1.0, 0.0));
		b2Transform_SetPosition(xf1, tmp);
		
		// Shape 1
		b2PolygonShape *sd1 = b2PolygonShape_Create();
		VECTOR p[3];
		mulx(&p[0], xf1, vec2(-30.0 / vars->m_physScale, 0.0));
		mulx(&p[1], xf1, vec2(30.0 / vars->m_physScale, 0.0));
		mulx(&p[2], xf1, vec2(0.0 , 15.0 / vars->m_physScale));
		b2PolygonShape_Set(sd1, p, 3);
		
		b2Transform *xf2 = b2Transform_Create();
		b2Transform_SetAngle(xf2, -0.3524 * PI);
		b2Transform_GetRotationMat(xf2, R);
		mulmv(&tmp, R, vec2(-30.0 / vars->m_physScale, 0.0));
		b2Transform_SetPosition(xf2, tmp);
		
		// Shape 2
		b2PolygonShape *sd2 = b2PolygonShape_Create();
		mulx(&p[0], xf2, vec2(-30.0 / vars->m_physScale, 0.0));
		mulx(&p[1], xf2, vec2(30.0 / vars->m_physScale, 0.0));
		mulx(&p[2], xf2, vec2(0.0 , 15.0 / vars->m_physScale));
		b2PolygonShape_Set(sd2, p, 3);
		
		body_def = b2BodyDef_Create();
		b2BodyDef_SetType(body_def, B2_DYNAMICBODY);
		b2BodyDef_SetFixedRotation(body_def, 1);
		
		int i = 0;
		for (i = 0; i < 5; i++)
		{
			x = 320.0 + random(6.0) - 3.0;
			b2BodyDef_SetPosition(body_def, vec2(x / vars->m_physScale, (-61.5 + 55.0 * -i + 300 ) / vars->m_physScale));
			b2BodyDef_SetAngle(body_def, 0.0);
			body = b2World_CreateBody(vars->m_world, body_def);
			b2Body_CreateFixture2(body, sd1, 2.0);
			b2Body_CreateFixture2(body, sd2, 2.0);
		}	
		
		// Destroy unused objects
		b2Transform_Destroy(xf1);
		b2Transform_Destroy(xf2);
		
		b2PolygonShape_Destroy(sd1);
		b2PolygonShape_Destroy(sd2);
		
		b2BodyDef_Destroy(body_def);
	}
	
	// Add container
	{
		b2PolygonShape *sd_bottom = b2PolygonShape_Create();
		b2PolygonShape_SetAsBox1(sd_bottom, 45.0 / vars->m_physScale, 4.5 / vars->m_physScale);
		
		b2PolygonShape *sd_left = b2PolygonShape_Create();
		b2PolygonShape_SetAsBox2(sd_left, 4.5 / vars->m_physScale, 81.0 / vars->m_physScale, vec2(-43.5 / vars->m_physScale, -70.5 / vars->m_physScale), -0.2);
		
		b2PolygonShape *sd_right = b2PolygonShape_Create();
		b2PolygonShape_SetAsBox2(sd_right, 4.5 / vars->m_physScale, 81.0 / vars->m_physScale, vec2(43.5 / vars->m_physScale, -70.5 / vars->m_physScale), 0.2);
		
		body_def = b2BodyDef_Create();
		b2BodyDef_SetType(body_def, B2_DYNAMICBODY);
		b2BodyDef_SetPosition(body_def, vec2(320.0 / vars->m_physScale, 300.0 / vars->m_physScale));
		
		body = b2World_CreateBody(vars->m_world, body_def);
		b2Body_CreateFixture2(body, sd_bottom, 4.0);
		b2Body_CreateFixture2(body, sd_left, 4.0);
		b2Body_CreateFixture2(body, sd_right, 4.0);
		
		// Destroy unused objects
		b2PolygonShape_Destroy(sd_bottom);
		b2PolygonShape_Destroy(sd_left);
		b2PolygonShape_Destroy(sd_right);
		
		b2BodyDef_Destroy(body_def);
	}
}

/// \brief Update current test suite
void compound_update()
{
	deftest_update(&rag_def_vars);
}

/// \brief Clean current test suite
void compound_exit()
{
	deftest_exit(&rag_def_vars);
}

#endif // __TEST_COMPOUND_H__
