#ifndef __TEST_CRANK__H__
#define __TEST_CRANK__H__

/// \brief default variables for crank test
DEF_VARIABLES crank_def_vars;

/// \brief Init crank test
void crank_init()
{
	deftest_init(&crank_def_vars);
	DEF_VARIABLES *vars = &crank_def_vars;
	
	// Set title
	str_cpy(str_test_name, "Slider Crank");
	
	b2BodyDef *ground_def = b2BodyDef_Create();
	b2BodyDef_SetPosition(ground_def, vec2(screen_size.x / 2 / vars->m_physScale, screen_size.y / vars->m_physScale));
	b2Body *ground = b2World_CreateBody(vars->m_world, ground_def);
	b2Body *body = NULL;
	b2PolygonShape *shape_def = NULL;
	b2BodyDef *body_def = NULL;
	b2FixtureDef *fixture_def = b2FixtureDef_Create();
	
	// Joints
	b2RevoluteJoint *rjoint_1 = NULL;
	b2PrismaticJoint *pjoint_1 = NULL;
		
	// Define crank
	shape_def = b2PolygonShape_Create();
	b2PolygonShape_SetAsBox1(shape_def, 7.5 / vars->m_physScale, 30.0 / vars->m_physScale);
	b2FixtureDef_SetShape(fixture_def, shape_def);
	b2FixtureDef_SetDensity(fixture_def, 1.0);
	
	b2RevoluteJointDef *rjoint_def = b2RevoluteJointDef_Create();
	
	b2Body *prev_body = ground;
	
	body_def = b2BodyDef_Create();
	b2BodyDef_SetType(body_def, B2_DYNAMICBODY);
	b2BodyDef_SetPosition(body_def, vec2(screen_size.x / 2 / vars->m_physScale, (360.0 - 105.0) / vars->m_physScale));
	body = b2World_CreateBody(vars->m_world, body_def);
	b2Body_CreateFixture(body, fixture_def); 
	
	b2RevoluteJointDef_Initialize(rjoint_def, prev_body, body, vec2(screen_size.x / 2 / vars->m_physScale, (360.0 - 75.0) / vars->m_physScale));
	b2RevoluteJointDef_SetMotorSpeed(rjoint_def, 1.0 * -PI);
	b2RevoluteJointDef_SetMaxMotorTorque(rjoint_def, 5000.0);
	b2RevoluteJointDef_SetEnableMotor(rjoint_def, 1);
	
	rjoint_1 = b2World_CreateJoint(vars->m_world, rjoint_def);
	
	// Delete unused objects
	b2BodyDef_Destroy(ground_def);
	b2PolygonShape_Destroy(shape_def);
	
	prev_body = body;
	
	// Define follower
	shape_def = b2PolygonShape_Create();
	b2PolygonShape_SetAsBox1(shape_def, 7.5 / vars->m_physScale, 60.0 / vars->m_physScale);
	b2FixtureDef_SetShape(fixture_def, shape_def);
	b2BodyDef_SetPosition(body_def, vec2(screen_size.x / 2 / vars->m_physScale, (360.0 - 195.0) / vars->m_physScale));
	body = b2World_CreateBody(vars->m_world, body_def);
	b2Body_CreateFixture(body, fixture_def);
	
	b2RevoluteJointDef_Initialize(rjoint_def, prev_body, body, vec2(screen_size.x / 2 / vars->m_physScale, (360.0 - 135.0) / vars->m_physScale));
	b2RevoluteJointDef_SetEnableMotor(rjoint_def, 0);
	b2World_CreateJoint(vars->m_world, rjoint_def);
	
	// Destroy unused objects
	b2PolygonShape_Destroy(shape_def);
	
	prev_body = body;
	
	// Define piston
	shape_def = b2PolygonShape_Create();
	b2PolygonShape_SetAsBox1(shape_def, 22.5 / vars->m_physScale, 22.5 / vars->m_physScale);
	b2FixtureDef_SetShape(fixture_def, shape_def);
	b2BodyDef_SetPosition(body_def, vec2(screen_size.x / 2 / vars->m_physScale, (360.0 - 255.0) / vars->m_physScale));
	body = b2World_CreateBody(vars->m_world, body_def);
	b2Body_CreateFixture(body, fixture_def);
	
	b2RevoluteJointDef_Initialize(rjoint_def, prev_body, body, vec2(screen_size.x / 2 / vars->m_physScale, (360.0 - 255.0) / vars->m_physScale));
	b2World_CreateJoint(vars->m_world, rjoint_def);
	
	b2PrismaticJointDef *pjoint_def = b2PrismaticJointDef_Create();
	b2PrismaticJointDef_Initialize(pjoint_def, ground, body, vec2(screen_size.x / 2 / vars->m_physScale, (360.0 - 255.0) / vars->m_physScale), vec2(0.0, 1.0));
	b2PrismaticJointDef_SetMaxMotorForce(pjoint_def, 500.0);
	b2PrismaticJointDef_SetEnableMotor(pjoint_def, 1);
	
	pjoint_1 = b2World_CreateJoint(vars->m_world, pjoint_def);
	
	// Destroy unused objects
	b2PolygonShape_Destroy(shape_def);
	b2RevoluteJointDef_Destroy(rjoint_def);
	b2PrismaticJointDef_Destroy(pjoint_def);
	
	// Create a payload
	shape_def = b2PolygonShape_Create();
	b2PolygonShape_SetAsBox1(shape_def, 22.5 / vars->m_physScale, 22.5 / vars->m_physScale);
	b2FixtureDef_SetShape(fixture_def, shape_def);
	b2FixtureDef_SetDensity(fixture_def, 2.0);
	b2BodyDef_SetPosition(body_def, vec2(screen_size.x / 2 / vars->m_physScale, (360.0 - 345.0) / vars->m_physScale));
	body = b2World_CreateBody(vars->m_world, body_def);
	b2Body_CreateFixture(body, fixture_def);
	
	// Destroy unused objects
	b2PolygonShape_Destroy(shape_def);
	b2BodyDef_Destroy(body_def);
}

/// \brief Update current test suite
void crank_update()
{
	deftest_update(&crank_def_vars);
}

/// \brief Clean current test suite
void crank_exit()
{
	deftest_exit(&crank_def_vars);
}

#endif // __TEST_CRANK__H__
