#ifndef __TEST_GEARS_H__
#define __TEST_GEARS_H__

/// \brief default variables for gears test
DEF_VARIABLES gears_def_vars;

/// \brief Init gears test
void gears_init()
{
	deftest_init(&gears_def_vars);
	DEF_VARIABLES *vars = &gears_def_vars;
	
	// Set title
	str_cpy(str_test_name, "Gears");
	
	b2BodyDef *ground_def = b2BodyDef_Create();
	b2BodyDef_SetPosition(ground_def, vec2(screen_size.x / 2 / vars->m_physScale, screen_size.y / vars->m_physScale));
	b2Body *ground = b2World_CreateBody(vars->m_world, ground_def);
	b2Body *body = NULL;
	b2Body *circleBody = NULL;
	b2PolygonShape *shape_def = NULL;
	b2BodyDef *body_def = NULL;
	b2FixtureDef *fixture_def = b2FixtureDef_Create();
	
	// Joints
	b2RevoluteJoint *rjoint_1 = NULL;
	
	b2RevoluteJoint *rjoint_2 = NULL;
	b2PrismaticJoint *pjoint_2 = NULL;
	b2GearJoint *gjoint_1 = NULL;
	b2GearJoint *gjoint_2 = NULL;
	
	b2CircleShape *circle1 = b2CircleShape_Create();
	b2CircleShape_SetRadius(circle1, 25 / vars->m_physScale);
	b2FixtureDef_SetShape(fixture_def, circle1);
	b2FixtureDef_SetDensity(fixture_def, 5.0);
	
	b2BodyDef *bd1 = b2BodyDef_Create();
	b2BodyDef_SetType(bd1, B2_DYNAMICBODY);
	b2BodyDef_SetPosition(bd1, vec2((screen_size.x / 2 - 75) / vars->m_physScale, 360 / 2 / vars->m_physScale));
	b2Body *body1 = b2World_CreateBody(vars->m_world, bd1);
	b2Body_CreateFixture(body1, fixture_def);
	
	b2RevoluteJointDef *jd1 = b2RevoluteJointDef_Create();
	VECTOR pos;
	b2BodyDef_GetPosition(bd1, &pos);
	b2RevoluteJointDef_Initialize(jd1, ground, body1, &pos);
	rjoint_1 = b2World_CreateJoint(vars->m_world, jd1);
	
	b2CircleShape *circle2 = b2CircleShape_Create();
	b2CircleShape_SetRadius(circle2, 50 / vars->m_physScale);
	b2FixtureDef_SetShape(fixture_def, circle2);
	b2FixtureDef_SetDensity(fixture_def, 5.0);
	
	b2BodyDef *bd2 = b2BodyDef_Create();
	b2BodyDef_SetType(bd2, B2_DYNAMICBODY);
	b2BodyDef_SetPosition(bd2, vec2(screen_size.x / 2 / vars->m_physScale, 360 / 2 / vars->m_physScale));
	b2Body *body2 = b2World_CreateBody(vars->m_world, bd2);
	b2Body_CreateFixture(body2, fixture_def);
	
	b2RevoluteJointDef *jd2 = b2RevoluteJointDef_Create();
	b2BodyDef_GetPosition(bd2, &pos);
	b2RevoluteJointDef_Initialize(jd2, ground, body2, &pos);
	rjoint_2 = b2World_CreateJoint(vars->m_world, jd2);
	
	b2PolygonShape *box = b2PolygonShape_Create();
	b2PolygonShape_SetAsBox1(box, 10 / vars->m_physScale, 100 / vars->m_physScale);
	b2FixtureDef_SetShape(fixture_def, box);
	b2FixtureDef_SetDensity(fixture_def, 5.0);
	
	b2BodyDef *bd3 = b2BodyDef_Create();
	b2BodyDef_SetType(bd3, B2_DYNAMICBODY);
	b2BodyDef_SetPosition(bd3, vec2((screen_size.x / 2 + 60) / vars->m_physScale, 360 / 2 / vars->m_physScale));
	b2Body *body3 = b2World_CreateBody(vars->m_world, bd3);
	b2Body_CreateFixture(body3, fixture_def);
	
	b2PrismaticJointDef *jd3 = b2PrismaticJointDef_Create();
	b2BodyDef_GetPosition(bd3, &pos);
	b2PrismaticJointDef_Initialize(jd3, ground, body3, &pos, vec2(0, 1));
	b2PrismaticJointDef_SetLowerTranslation(jd3, -25.0 / vars->m_physScale);
	b2PrismaticJointDef_SetUpperTranslation(jd3, 100 / vars->m_physScale);
	b2PrismaticJointDef_SetEnableLimit(jd3, 1);
	
	pjoint_2 = b2World_CreateJoint(vars->m_world, jd3);
	
	b2GearJointDef *jd4 = b2GearJointDef_Create();
	b2GearJointDef_SetBodyA(jd4, body1);
	b2GearJointDef_SetBodyB(jd4, body2);
	b2GearJointDef_SetJoint1(jd4, rjoint_1);
	b2GearJointDef_SetJoint2(jd4, rjoint_2);
	b2GearJointDef_SetRatio(jd4, b2CircleShape_GetRadius(circle2) / b2CircleShape_GetRadius(circle1));
	
	gjoint_1 = b2World_CreateJoint(vars->m_world, jd4);
	
	b2GearJointDef *jd5 = b2GearJointDef_Create();
	b2GearJointDef_SetBodyA(jd5, body2);
	b2GearJointDef_SetBodyB(jd5, body3);
	b2GearJointDef_SetJoint1(jd5, rjoint_2);
	b2GearJointDef_SetJoint2(jd5, pjoint_2);
	b2GearJointDef_SetRatio(jd5, -1.0 / b2CircleShape_GetRadius(circle2));
	
	gjoint_2 = b2World_CreateJoint(vars->m_world, jd5);
	
	// Destroy unused objects
	b2CircleShape_Destroy(circle1);
	b2CircleShape_Destroy(circle2);
	b2PolygonShape_Destroy(box);
	// BodyDefs
	b2BodyDef_Destroy(ground_def);
	b2BodyDef_Destroy(bd1);
	b2BodyDef_Destroy(bd2);
	b2BodyDef_Destroy(bd3);
	// JointDefs
	b2JointDef_Destroy(jd1);
	b2JointDef_Destroy(jd2);
	b2JointDef_Destroy(jd3);
	b2JointDef_Destroy(jd4);
	b2JointDef_Destroy(jd5);
}

/// \brief Update current test suite
void gears_update()
{
	deftest_update(&gears_def_vars);
}

/// \brief Clean current test suite
void gears_exit()
{
	deftest_exit(&gears_def_vars);
}

#endif // __TEST_GEARS_H__
