#ifndef __TEST_PULLEY_H__
#define __TEST_PULLEY_H__

/// \brief default variables for pulley test
DEF_VARIABLES pulley_def_vars;

/// \brief Init pulley test
void pulley_init()
{
	deftest_init(&pulley_def_vars);
	DEF_VARIABLES *vars = &pulley_def_vars;
	
	// Set title
	str_cpy(str_test_name, "Pulley");
	
	{
		b2BodyDef *ground_def = b2BodyDef_Create();	
		b2Body *ground = b2World_CreateBody(vars->m_world, ground_def);
	
		b2PolygonShape *shape = b2PolygonShape_Create();
		b2PolygonShape_SetAsEdge(shape, vec2((screen_size.x - 200) / 2 / vars->m_physScale, (screen_size.y - 150) / 2 / vars->m_physScale), vec2((screen_size.x + 200) / 2 / vars->m_physScale, (screen_size.y - 150) / 2 / vars->m_physScale));
		//b2Body_CreateFixture2(ground, shape, 0.0);
		
		// Destroy unused objects
		b2BodyDef_Destroy(ground_def);
		b2PolygonShape_Destroy(shape);
	}
	
	b2PulleyJoint *pjoint1 = NULL;
	{
		b2PolygonShape *shape = b2PolygonShape_Create();
		b2PolygonShape_SetAsBox1(shape, 20 / vars->m_physScale, 40 / vars->m_physScale );	
		
		b2BodyDef *bd = b2BodyDef_Create();
		b2BodyDef_SetType(bd, B2_DYNAMICBODY);
		b2BodyDef_SetPosition(bd, vec2((screen_size.x - 200) / 2 / vars->m_physScale, (screen_size.y + 100) / 2 / vars->m_physScale));
		b2Body *body1 = b2World_CreateBody(vars->m_world, bd);
		b2Body_CreateFixture2(body1, shape, 5.0);
		
		b2BodyDef_SetPosition(bd, vec2((screen_size.x + 200) / 2 / vars->m_physScale, (screen_size.y + 100) / 2 / vars->m_physScale));
		b2Body *body2 = b2World_CreateBody(vars->m_world, bd);
		b2Body_CreateFixture2(body2, shape, 5.0);
		
		b2PulleyJointDef *pulley_def = b2PulleyJointDef_Create();
		VECTOR anchor1;
		vec_set(&anchor1, vec2((screen_size.x - 200) / 2 / vars->m_physScale, (screen_size.y + 100 - 80) / 2 / vars->m_physScale));
		
		VECTOR anchor2;
		vec_set(&anchor2, vec2((screen_size.x + 200) / 2 / vars->m_physScale, (screen_size.y + 100 - 80) / 2 / vars->m_physScale));
		
		VECTOR groundAnchor1;
		vec_set(&groundAnchor1, vec2((screen_size.x - 200) / 2 / vars->m_physScale, (screen_size.y + 100 - 250) / 2 / vars->m_physScale));
		
		VECTOR groundAnchor2;
		vec_set(&groundAnchor2, vec2((screen_size.x + 200) / 2 / vars->m_physScale, (screen_size.y + 100 - 250) / 2 / vars->m_physScale));
		
		b2PulleyJointDef_Initialize(pulley_def, body1, body2, &groundAnchor1, &groundAnchor2, &anchor1, &anchor2, 2.0);
		
		pjoint1 = b2World_CreateJoint(vars->m_world, pulley_def);
		
		// Destroy unused objects
		b2PolygonShape_Destroy(shape);
		b2BodyDef_Destroy(bd);
		b2PulleyJointDef_Destroy(pulley_def);
	}
}

/// \brief Update current test suite
void pulley_update()
{
	deftest_update(&pulley_def_vars);
}

/// \brief Clean current test suite
void spulley_exit()
{
	deftest_exit(&pulley_def_vars);
}

#endif // __TEST_PULLEY_H__
