#ifndef __TEST_RAGDOLL_H__
#define __TEST_RAGDOLL_H__

#define RAGDOLL_COUNT 2

/// \brief default variables for ragdoll test
DEF_VARIABLES rag_def_vars;

/// \brief Init ragdoll test
void ragdoll_init()
{
	deftest_init(&rag_def_vars);
	DEF_VARIABLES *vars = &rag_def_vars;
	
	// Set title
	str_cpy(str_test_name, "Ragdoll");
	
	// Local variables
	b2BodyDef *body_def = b2BodyDef_Create();
	b2RevoluteJointDef *joint_def = b2RevoluteJointDef_Create();
	b2FixtureDef *fixture_def = b2FixtureDef_Create();
	
	b2CircleShape *circle_shape = NULL;
	b2PolygonShape *box_shape = NULL;
		
	// Add ragdolls
	int i = 0;
	for (i = 0; i < RAGDOLL_COUNT; i++)
	{
		random_seed(0);
		var start_x = 70 + random(20) + 480 * i;
		var start_y = 20 + random(50);
		
		// BODIES
		// Set to dynamic bodies
		b2BodyDef_SetType(body_def, B2_DYNAMICBODY);
		
		//////////////////////////////
		// Head
		//////////////////////////////
		circle_shape = b2CircleShape_Create();
		b2CircleShape_SetRadius(circle_shape, 12.5 / vars->m_physScale);
		
		b2FixtureDef_SetShape(fixture_def, circle_shape);
		b2FixtureDef_SetDensity(fixture_def, 1.0);
		b2FixtureDef_SetFriction(fixture_def, 0.4);
		b2FixtureDef_SetRestitution(fixture_def, 0.3);
		
		b2BodyDef_SetPosition(body_def, vec2(start_x / vars->m_physScale, start_y / vars->m_physScale));
		b2Body *head = b2World_CreateBody(vars->m_world, body_def);
		b2Body_CreateFixture(head, fixture_def);
		
		VECTOR outImpulse;
		b2Body_GetWorldCenter(head, &outImpulse);
		b2Body_ApplyLinearImpulse(head, vec2(random(100) - 50, random(100) - 50), &outImpulse);
		// Destroy b2CircleShape
		b2CircleShape_Destroy(circle_shape);
		
		//////////////////////////////
		// Torso 1
		//////////////////////////////
		box_shape = b2PolygonShape_Create();
		b2PolygonShape_SetAsBox1(box_shape, 15 / vars->m_physScale, 10 / vars->m_physScale);
		
		b2FixtureDef_SetShape(fixture_def, box_shape);
		b2FixtureDef_SetDensity(fixture_def, 1.0);
		b2FixtureDef_SetFriction(fixture_def, 0.4);
		b2FixtureDef_SetRestitution(fixture_def, 0.1);
		
		b2BodyDef_SetPosition(body_def, vec2(start_x / vars->m_physScale, (start_y + 28) / vars->m_physScale));
		b2Body *torso1 = b2World_CreateBody(vars->m_world, body_def);
		b2Body_CreateFixture(torso1, fixture_def);
		// Destroy Shape
		b2PolygonShape_Destroy(box_shape);
		
		//////////////////////////////
		// Torso 2
		//////////////////////////////
		box_shape = b2PolygonShape_Create();
		b2PolygonShape_SetAsBox1(box_shape, 15 / vars->m_physScale, 10 / vars->m_physScale);
		
		b2FixtureDef_SetShape(fixture_def, box_shape);
		
		b2BodyDef_SetPosition(body_def, vec2(start_x / vars->m_physScale, (start_y + 43) / vars->m_physScale));
		b2Body *torso2 = b2World_CreateBody(vars->m_world, body_def);
		b2Body_CreateFixture(torso2, fixture_def);
		// Destroy Shape
		b2PolygonShape_Destroy(box_shape);
		
		//////////////////////////////
		// Torso 3
		//////////////////////////////
		box_shape = b2PolygonShape_Create();
		b2PolygonShape_SetAsBox1(box_shape, 15 / vars->m_physScale, 10 / vars->m_physScale);
		
		b2FixtureDef_SetShape(fixture_def, box_shape);
		
		b2BodyDef_SetPosition(body_def, vec2(start_x / vars->m_physScale, (start_y + 58) / vars->m_physScale));
		b2Body *torso3 = b2World_CreateBody(vars->m_world, body_def);
		b2Body_CreateFixture(torso3, fixture_def);
		// Destroy Shape
		b2PolygonShape_Destroy(box_shape);
		
		//////////////////////////////
		// Upper arm
		//////////////////////////////
		b2FixtureDef_SetDensity(fixture_def, 1.0);
		b2FixtureDef_SetFriction(fixture_def, 1.4);
		b2FixtureDef_SetRestitution(fixture_def, 0.1);
		// L
		box_shape = b2PolygonShape_Create();
		b2PolygonShape_SetAsBox1(box_shape, 18 / vars->m_physScale, 6.5 / vars->m_physScale);
		
		b2FixtureDef_SetShape(fixture_def, box_shape);
		
		b2BodyDef_SetPosition(body_def, vec2((start_x - 30) / vars->m_physScale, (start_y + 20) / vars->m_physScale));
		b2Body *upperArmL = b2World_CreateBody(vars->m_world, body_def);
		b2Body_CreateFixture(upperArmL, fixture_def);
		// Destroy Shape
		b2PolygonShape_Destroy(box_shape);
		
		// R
		box_shape = b2PolygonShape_Create();
		b2PolygonShape_SetAsBox1(box_shape, 18 / vars->m_physScale, 6.5 / vars->m_physScale);
		
		b2FixtureDef_SetShape(fixture_def, box_shape);
		
		b2BodyDef_SetPosition(body_def, vec2((start_x + 30) / vars->m_physScale, (start_y + 20) / vars->m_physScale));
		b2Body *upperArmR = b2World_CreateBody(vars->m_world, body_def);
		b2Body_CreateFixture(upperArmR, fixture_def);
		// Destroy Shape
		b2PolygonShape_Destroy(box_shape);
		
		//////////////////////////////
		// Lower arm
		//////////////////////////////
		b2FixtureDef_SetDensity(fixture_def, 1.0);
		b2FixtureDef_SetFriction(fixture_def, 0.4);
		b2FixtureDef_SetRestitution(fixture_def, 0.1);
		// L
		box_shape = b2PolygonShape_Create();
		b2PolygonShape_SetAsBox1(box_shape, 17 / vars->m_physScale, 6 / vars->m_physScale);
		
		b2FixtureDef_SetShape(fixture_def, box_shape);
		
		b2BodyDef_SetPosition(body_def, vec2((start_x - 57) / vars->m_physScale, (start_y + 20) / vars->m_physScale));
		b2Body *lowerArmL = b2World_CreateBody(vars->m_world, body_def);
		b2Body_CreateFixture(lowerArmL, fixture_def);
		// Destroy Shape
		b2PolygonShape_Destroy(box_shape);
		
		// R
		box_shape = b2PolygonShape_Create();
		b2PolygonShape_SetAsBox1(box_shape, 17 / vars->m_physScale, 6 / vars->m_physScale);
		
		b2FixtureDef_SetShape(fixture_def, box_shape);
		
		b2BodyDef_SetPosition(body_def, vec2((start_x + 57) / vars->m_physScale, (start_y + 20) / vars->m_physScale));
		b2Body *lowerArmR = b2World_CreateBody(vars->m_world, body_def);
		b2Body_CreateFixture(lowerArmR, fixture_def);
		// Destroy Shape
		b2PolygonShape_Destroy(box_shape);
		
		//////////////////////////////
		// Upper leg
		//////////////////////////////
		b2FixtureDef_SetDensity(fixture_def, 1.0);
		b2FixtureDef_SetFriction(fixture_def, 0.4);
		b2FixtureDef_SetRestitution(fixture_def, 0.1);
		// L
		box_shape = b2PolygonShape_Create();
		b2PolygonShape_SetAsBox1(box_shape, 7.5 / vars->m_physScale, 22 / vars->m_physScale);
		
		b2FixtureDef_SetShape(fixture_def, box_shape);
		
		b2BodyDef_SetPosition(body_def, vec2((start_x - 8) / vars->m_physScale, (start_y + 85) / vars->m_physScale));
		b2Body *upperLegL = b2World_CreateBody(vars->m_world, body_def);
		b2Body_CreateFixture(upperLegL, fixture_def);
		// Destroy Shape
		b2PolygonShape_Destroy(box_shape);
		
		// R
		box_shape = b2PolygonShape_Create();
		b2PolygonShape_SetAsBox1(box_shape, 7.5 / vars->m_physScale, 22 / vars->m_physScale);
		
		b2FixtureDef_SetShape(fixture_def, box_shape);
		
		b2BodyDef_SetPosition(body_def, vec2((start_x + 8) / vars->m_physScale, (start_y + 85) / vars->m_physScale));
		b2Body *upperLegR = b2World_CreateBody(vars->m_world, body_def);
		b2Body_CreateFixture(upperLegR, fixture_def);
		// Destroy Shape
		b2PolygonShape_Destroy(box_shape);

		//////////////////////////////
		// Lower leg
		//////////////////////////////
		b2FixtureDef_SetDensity(fixture_def, 1.0);
		b2FixtureDef_SetFriction(fixture_def, 0.4);
		b2FixtureDef_SetRestitution(fixture_def, 0.1);
		// L
		box_shape = b2PolygonShape_Create();
		b2PolygonShape_SetAsBox1(box_shape, 6 / vars->m_physScale, 20 / vars->m_physScale);
		
		b2FixtureDef_SetShape(fixture_def, box_shape);
		
		b2BodyDef_SetPosition(body_def, vec2((start_x - 8) / vars->m_physScale, (start_y + 120) / vars->m_physScale));
		b2Body *lowerLegL = b2World_CreateBody(vars->m_world, body_def);
		b2Body_CreateFixture(lowerLegL, fixture_def);
		// Destroy Shape
		b2PolygonShape_Destroy(box_shape);
		
		// R
		box_shape = b2PolygonShape_Create();
		b2PolygonShape_SetAsBox1(box_shape, 6 / vars->m_physScale, 20 / vars->m_physScale);
		
		b2FixtureDef_SetShape(fixture_def, box_shape);
		
		b2BodyDef_SetPosition(body_def, vec2((start_x + 8) / vars->m_physScale, (start_y + 120) / vars->m_physScale));
		b2Body *lowerLegR = b2World_CreateBody(vars->m_world, body_def);
		b2Body_CreateFixture(lowerLegR, fixture_def);
		// Destroy Shape
		b2PolygonShape_Destroy(box_shape);
		
		
		// JOINTS
		b2RevoluteJointDef_SetEnableLimit(joint_def, 1);
		
		// Head to shoulders
		b2RevoluteJointDef_SetLowerAngle(joint_def, -40 / (180/PI));
		b2RevoluteJointDef_SetUpperAngle(joint_def, 40 / (180/PI));
		b2RevoluteJointDef_Initialize(joint_def, torso1, head, vec2(start_x / vars->m_physScale, (start_y + 15) / vars->m_physScale));
		b2World_CreateJoint(vars->m_world, joint_def);
		
		// Upper arm to shoulders
		// L
		b2RevoluteJointDef_SetLowerAngle(joint_def, -85 / (180/PI));
		b2RevoluteJointDef_SetUpperAngle(joint_def, 130 / (180/PI));
		b2RevoluteJointDef_Initialize(joint_def, torso1, upperArmL, vec2((start_x - 18) / vars->m_physScale, (start_y + 20) / vars->m_physScale));
		b2World_CreateJoint(vars->m_world, joint_def);
		// R
		b2RevoluteJointDef_SetLowerAngle(joint_def, -130 / (180/PI));
		b2RevoluteJointDef_SetUpperAngle(joint_def, 85 / (180/PI));
		b2RevoluteJointDef_Initialize(joint_def, torso1, upperArmR, vec2((start_x + 18) / vars->m_physScale, (start_y + 20) / vars->m_physScale));
		b2World_CreateJoint(vars->m_world, joint_def);
		
		// Lower arm to upper arm
		// L
		b2RevoluteJointDef_SetLowerAngle(joint_def, -130 / (180/PI));
		b2RevoluteJointDef_SetUpperAngle(joint_def, 10 / (180/PI));
		b2RevoluteJointDef_Initialize(joint_def, upperArmL, lowerArmL, vec2((start_x - 45) / vars->m_physScale, (start_y + 20) / vars->m_physScale));
		b2World_CreateJoint(vars->m_world, joint_def);
		// R
		b2RevoluteJointDef_SetLowerAngle(joint_def, -10 / (180/PI));
		b2RevoluteJointDef_SetUpperAngle(joint_def, 130 / (180/PI));
		b2RevoluteJointDef_Initialize(joint_def, upperArmR, lowerArmR, vec2((start_x + 45) / vars->m_physScale, (start_y + 20) / vars->m_physScale));
		b2World_CreateJoint(vars->m_world, joint_def);
		
		// Shoulders/stomach
		b2RevoluteJointDef_SetLowerAngle(joint_def, -15 / (180/PI));
		b2RevoluteJointDef_SetUpperAngle(joint_def, 15 / (180/PI));
		b2RevoluteJointDef_Initialize(joint_def, torso1, torso2, vec2(start_x / vars->m_physScale, (start_y + 35) / vars->m_physScale));
		b2World_CreateJoint(vars->m_world, joint_def);
		// Stomach/hips
		b2RevoluteJointDef_Initialize(joint_def, torso2, torso3, vec2(start_x / vars->m_physScale, (start_y + 50) / vars->m_physScale));
		b2World_CreateJoint(vars->m_world, joint_def);
		
		// Torso to upper leg
		// L
		b2RevoluteJointDef_SetLowerAngle(joint_def, -25 / (180/PI));
		b2RevoluteJointDef_SetUpperAngle(joint_def, 45 / (180/PI));
		b2RevoluteJointDef_Initialize(joint_def, torso3, upperLegL, vec2((start_x - 8) / vars->m_physScale, (start_y + 72) / vars->m_physScale));
		b2World_CreateJoint(vars->m_world, joint_def);
		// R
		b2RevoluteJointDef_SetLowerAngle(joint_def, -45 / (180/PI));
		b2RevoluteJointDef_SetUpperAngle(joint_def, 25 / (180/PI));
		b2RevoluteJointDef_Initialize(joint_def, torso3, upperLegR, vec2((start_x + 8) / vars->m_physScale, (start_y + 72) / vars->m_physScale));
		b2World_CreateJoint(vars->m_world, joint_def);
		
		// Upper leg to lower leg
		// L
		b2RevoluteJointDef_SetLowerAngle(joint_def, -25 / (180/PI));
		b2RevoluteJointDef_SetUpperAngle(joint_def, 115 / (180/PI));
		b2RevoluteJointDef_Initialize(joint_def, upperLegL, lowerLegL, vec2((start_x - 8) / vars->m_physScale, (start_y + 105) / vars->m_physScale));
		b2World_CreateJoint(vars->m_world, joint_def);
		// R
		b2RevoluteJointDef_SetLowerAngle(joint_def, -115 / (180/PI));
		b2RevoluteJointDef_SetUpperAngle(joint_def, 25 / (180/PI));
		b2RevoluteJointDef_Initialize(joint_def, upperLegR, lowerLegR, vec2((start_x + 8) / vars->m_physScale, (start_y + 105) / vars->m_physScale));
		b2World_CreateJoint(vars->m_world, joint_def);
	}
	
	// Add stairs on the left
	b2BodyDef_SetType(body_def, B2_STATICBODY);
	b2FixtureDef_SetDensity(fixture_def, 0.0);
	b2FixtureDef_SetFriction(fixture_def, 0.4);
	b2FixtureDef_SetRestitution(fixture_def, 0.3);
	int j = 0;
	for (j = 1; j <= 10; j++)
	{
		box_shape = b2PolygonShape_Create();
		b2PolygonShape_SetAsBox1(box_shape, (10 * j) / vars->m_physScale, 10 / vars->m_physScale);
		b2FixtureDef_SetShape(fixture_def, box_shape);
		
		b2BodyDef_SetPosition(body_def, vec2((10*j) / vars->m_physScale, (150 + 20 * j) / vars->m_physScale));
		b2Body *stair = b2World_CreateBody(vars->m_world, body_def);
		b2Body_CreateFixture(stair, fixture_def);
		
		// Destroy shape
		b2PolygonShape_Destroy(box_shape);
	}
	
	// Add stairs on the right
	int k = 0;
	for (k = 1; k <= 10; k++)
	{
		box_shape = b2PolygonShape_Create();
		b2PolygonShape_SetAsBox1(box_shape, (10 * k) / vars->m_physScale, 10 / vars->m_physScale);
		b2FixtureDef_SetShape(fixture_def, box_shape);
		
		b2BodyDef_SetPosition(body_def, vec2((640-10*k) / vars->m_physScale, (150 + 20 * k) / vars->m_physScale));
		b2Body *stair = b2World_CreateBody(vars->m_world, body_def);
		b2Body_CreateFixture(stair, fixture_def);
		
		// Destroy shape
		b2PolygonShape_Destroy(box_shape);
	}
	
	box_shape = b2PolygonShape_Create();
	b2PolygonShape_SetAsBox1(box_shape, 30 / vars->m_physScale, 40 / vars->m_physScale);
	b2FixtureDef_SetShape(fixture_def, box_shape);
	
	b2BodyDef_SetPosition(body_def, vec2(320 / vars->m_physScale, 320 / vars->m_physScale));
	b2Body *stair = b2World_CreateBody(vars->m_world, body_def);
	b2Body_CreateFixture(stair, fixture_def);
	
	// Destroy shape
	b2PolygonShape_Destroy(box_shape);
	
	//////////////////////////////
	// Destroy all unused shapes
	//////////////////////////////
	b2BodyDef_Destroy(body_def);
	b2RevoluteJointDef_Destroy(joint_def);
	b2FixtureDef_Destroy(fixture_def);
}

/// \brief Update current test suite
void ragdoll_update()
{
	deftest_update(&rag_def_vars);
}

/// \brief Clean current test suite
void ragdoll_exit()
{
	deftest_exit(&rag_def_vars);
}

#endif // __TEST_RAGDOLL_H__
