#ifndef __TEST_STACKED_BOX_H__
#define __TEST_STACKED_BOX_H__

/// \brief default variables for stacked box test
DEF_VARIABLES stbox_def_vars;

#define SB_COLUMN_COUNT 5
#define SB_ROW_COUNT 14

#define SB_X_START 300
#define SB_X_SPACE 50

// Bullet instance
b2Body *sb_bullet = NULL;

// Instruction panel
FONT *fnt_sb_instruction = "Arial#16";
TEXT *sb_pan_instruction = 
{
	string(
				"Press (space) to fire a bullet."
			);	
	
	font = fnt_sb_instruction;
		
	blue = 0;
	green = 0;
	red = 255;
	
	pos_x = 10;
	pos_y = 20; 
}

// Last space state
var sb_last_space = 0;

/// \brief Init stacked box test
void stacked_box_init()
{
	deftest_init(&stbox_def_vars);
	DEF_VARIABLES *vars = &stbox_def_vars;
	
	// Set title
	str_cpy(str_test_name, "Vertical Stack");
	
	/*b2BodyDef *ground_def = b2BodyDef_Create();
	b2BodyDef_SetPosition(ground_def, vec2(screen_size.x / 2 / vars->m_physScale, screen_size.y / vars->m_physScale));
	b2Body *ground = b2World_CreateBody(vars->m_world, ground_def);*/
	
	b2Body *body = NULL;
	
	b2BodyDef *body_def = b2BodyDef_Create();
	b2BodyDef_SetType(body_def, B2_DYNAMICBODY);
	
	b2PolygonShape *shape_def = b2PolygonShape_Create();
	b2PolygonShape_SetAsBox1(shape_def, 10 / vars->m_physScale, 10 / vars->m_physScale);
	
	b2FixtureDef *fixture_def = b2FixtureDef_Create();
	b2FixtureDef_SetDensity(fixture_def, 1.0);
	b2FixtureDef_SetFriction(fixture_def, 0.3);
	b2FixtureDef_SetRestitution(fixture_def, 0.1);
	b2FixtureDef_SetShape(fixture_def, shape_def);
	
	int i = 0;
	int j = 0;
	
	// Create stacks	
	for (j = 0; j < SB_COLUMN_COUNT; j++ )
	{
		var pos_x = SB_X_START + (SB_X_SPACE * j);
		
		for (i = 0; i < SB_ROW_COUNT; i++)
		{
			int n = j * SB_ROW_COUNT + i;
			
			// Position bodyDef
			b2BodyDef_SetPosition(body_def, vec2(pos_x / vars->m_physScale, (360 - 5 - i * 25 ) / vars->m_physScale));
			// Create body & fixture
			body = b2World_CreateBody(vars->m_world, body_def);
			b2Body_CreateFixture(body, fixture_def);
		}
	}
	
	// Destroy unused objects
	b2BodyDef_Destroy(body_def);
	b2PolygonShape_Destroy(shape_def);
	b2FixtureDef_Destroy(fixture_def);
	
	// Show instruction panel
	sb_pan_instruction->flags |= SHOW;
}

/// \brief Update current test suite
void stacked_box_update()
{
	deftest_update(&stbox_def_vars);
	DEF_VARIABLES *vars = &stbox_def_vars;
		
	// Fire bullet
	if (key_space == 1 && sb_last_space == 0)
	{
		if (sb_bullet != NULL)
		{
			b2World_DestroyBody(vars->m_world, sb_bullet);
			sb_bullet = NULL;
		}
		
		b2CircleShape *bullet_shape = b2CircleShape_Create();
		b2CircleShape_SetRadius(bullet_shape, 2 / vars->m_physScale);
		
		b2FixtureDef *bullet_fixture = b2FixtureDef_Create();
		b2FixtureDef_SetShape(bullet_fixture, bullet_shape);
		b2FixtureDef_SetDensity(bullet_fixture, 100.0);
		b2FixtureDef_SetRestitution(bullet_fixture, 0.05);
		
		b2BodyDef *bullet_body = b2BodyDef_Create();
		b2BodyDef_SetType(bullet_body, B2_DYNAMICBODY);
		b2BodyDef_SetBullet(bullet_body, 1);
		b2BodyDef_SetPosition(bullet_body, vec2(-55 / vars->m_physScale, (screen_size.y / 2 + random(150)) / vars->m_physScale));
		
		sb_bullet = b2World_CreateBody(vars->m_world, bullet_body);
		b2Body_CreateFixture(sb_bullet, bullet_fixture);
		
		b2Body_SetLinearVelocity(sb_bullet, vec2(400.0, 0.0));
		
		// Destroy unused objects
		b2CircleShape_Destroy(bullet_shape);
		b2FixtureDef_Destroy(bullet_fixture);
		b2BodyDef_Destroy(bullet_body);
	}
	// Store last space status
	sb_last_space = key_space;
	
}

/// \brief Clean current test suite
void stacked_box_exit()
{
	// Default clean
	deftest_exit(&stbox_def_vars);
	sb_bullet = NULL;
	
	// Hide instruction panel
	sb_pan_instruction->flags &= ~SHOW;
}

#endif // __TEST_STACKED_BOX_H__
