#ifndef __TEST_THEO_JANSEN_H__
#define __TEST_THEO_JANSEN_H__

/// \brief default variables for theo jansen test
DEF_VARIABLES theojansen_def_vars;

// Test variables
var tj_scale = 0;
VECTOR tj_offset;
var tj_motor_on = 1;
var tj_motor_speed; 

b2Body *tj_chassis = NULL;
b2Body *tj_wheel = NULL;

b2RevoluteJoint *tj_motor_joint = NULL;

// Instruction panel
FONT *fnt_tj_instruction = "Arial#16";
TEXT *tj_pan_instruction = 
{
	string(
				"Keys : left = a, break = s, right = d, toggle_motor = m"
			);	
	
	font = fnt_sb_instruction;
		
	blue = 0;
	green = 0;
	red = 255;
	
	pos_x = 10;
	pos_y = 20; 
}

// Create leg function prototype
void theojansen_create_leg(var s, VECTOR *wheel_anchor);

// Last key states
var tj_last_a = 0;
var tj_last_s = 0;
var tj_last_d = 0;
var tj_last_m = 0;

/// \brief Init stacked box test
void theojansen_init()
{
	deftest_init(&theojansen_def_vars);
	DEF_VARIABLES *vars = &theojansen_def_vars;
	
	// Set title
	str_cpy(str_test_name, "Theo Jansen Walker");
		
	// Scale walker by variable
	tj_scale = vars->m_physScale * 2;	
	
	// Set position in world space
	vec_set(&tj_offset, vec2(120 / vars->m_physScale, 250 / vars->m_physScale));
	tj_motor_speed = -2.0;
	tj_motor_on = 1;
	
	VECTOR pivot;
	vec_set(&pivot, vec2(0.0, -24 / tj_scale));
	
	b2PolygonShape *pd = NULL;
	b2CircleShape *cd = NULL;
	b2FixtureDef *fd = NULL;
	b2BodyDef *bd = NULL;
	b2Body *body = NULL;
	
	// Create shapes	
	cd = b2CircleShape_Create();
	b2CircleShape_SetRadius(cd, 7.5 / tj_scale);	

	bd = b2BodyDef_Create();
	b2BodyDef_SetType(bd, B2_DYNAMICBODY);

	int i = 0;
	for (i = 0; i < 40; i++)
	{
		// Position in world space
		b2BodyDef_SetPosition(bd, vec2((random(620) + 10) / vars->m_physScale, 350 / vars->m_physScale));
		
		body = b2World_CreateBody(vars->m_world, bd);
		b2Body_CreateFixture2(body, cd, 1.0);
	}
	
	// Destroy unused objects
	b2CircleShape_Destroy(cd);
	b2BodyDef_Destroy(bd);
	
	{
		pd = b2PolygonShape_Create();
		b2PolygonShape_SetAsBox1(pd, 75 / tj_scale, 30 / tj_scale);
	
		fd = b2FixtureDef_Create();
		b2FixtureDef_SetShape(fd, pd);
		b2FixtureDef_SetDensity(fd, 1.0);
		
		b2Filter *filter = b2FixtureDef_GetFilter(fd);
		filter->groupIndex = -1;
		
		bd = b2BodyDef_Create();
		b2BodyDef_SetType(bd, B2_DYNAMICBODY);
		
		VECTOR temp;
		vec_set(&temp, &pivot);
		vec_add(&temp, &tj_offset);
		b2BodyDef_SetPosition(bd, &temp);
		
		tj_chassis = b2World_CreateBody(vars->m_world, bd);
		b2Body_CreateFixture(tj_chassis, fd);
		
		// Destroy unused objects
		b2BodyDef_Destroy(bd);
		b2PolygonShape_Destroy(pd);
		b2FixtureDef_Destroy(fd);
	}
	
	{
		cd = b2CircleShape_Create();
		b2CircleShape_SetRadius(cd, 48 / tj_scale);
		
		fd = b2FixtureDef_Create();
		
		b2FixtureDef_SetShape(fd, cd);
		b2FixtureDef_SetDensity(fd, 1.0);
		
		b2Filter *filter = b2FixtureDef_GetFilter(fd);
		filter->groupIndex = -1;
		
		bd = b2BodyDef_Create();
		b2BodyDef_SetType(bd, B2_DYNAMICBODY);
		
		VECTOR temp;
		vec_set(&temp, &pivot);
		vec_add(&temp, &tj_offset);
		b2BodyDef_SetPosition(bd, &temp);
		
		tj_wheel = b2World_CreateBody(vars->m_world, bd);
		b2Body_CreateFixture(tj_wheel, fd);
		
		// Destroy unused objects
		b2CircleShape_Destroy(cd);
		b2FixtureDef_Destroy(fd);
		b2BodyDef_Destroy(bd);
	}
	
	{
		b2RevoluteJointDef *jd = b2RevoluteJointDef_Create();
		VECTOR po;
		vec_set(&po, &pivot);
		vec_add(&po, &tj_offset);
		
		b2RevoluteJointDef_Initialize(jd, tj_wheel, tj_chassis, &po);
		b2JointDef_SetCollideConnected(jd, 0);
		b2RevoluteJointDef_SetMotorSpeed(jd, tj_motor_speed);
		b2RevoluteJointDef_SetMaxMotorTorque(jd, 400.0);
		b2RevoluteJointDef_SetEnableMotor(jd, tj_motor_on);
		
		tj_motor_joint = b2World_CreateJoint(vars->m_world, jd);
		
		// Destroy unused objects
		b2RevoluteJointDef_Destroy(jd);
	}
	
	VECTOR wheel_anchor;
	vec_set(&wheel_anchor, vec2(0.0, 24.0 / tj_scale));
	vec_add(&wheel_anchor, &pivot);
	
	theojansen_create_leg(-1.0, wheel_anchor);
	theojansen_create_leg(1.0, wheel_anchor);
	
	VECTOR wheel_pos;
	
	b2Body_GetPosition(tj_wheel, &wheel_pos);
	b2Body_SetTransform(tj_wheel, &wheel_pos, 120.0 * PI / 180);
	theojansen_create_leg(-1.0, wheel_anchor);
	theojansen_create_leg(1.0, wheel_anchor);
	
	b2Body_GetPosition(tj_wheel, &wheel_pos);
	b2Body_SetTransform(tj_wheel, &wheel_pos, -120.0 * PI / 180);
	theojansen_create_leg(-1.0, wheel_anchor);
	theojansen_create_leg(1.0, wheel_anchor);
	
	// Show instruction panel
	tj_pan_instruction->flags |= SHOW;
}

/// \brief Update current test suite
void theojansen_update()
{
	deftest_update(&theojansen_def_vars);
	DEF_VARIABLES *vars = &theojansen_def_vars;
	
	// Handle keys
	if (key_a == 1 && tj_last_a == 0)
	{
		b2Body_SetAwake(tj_chassis, 1);
		b2RevoluteJoint_SetMotorSpeed(tj_motor_joint, -tj_motor_speed);
	}
	
	if (key_s == 1 && tj_last_s == 0)
	{
		b2Body_SetAwake(tj_chassis, 1);
		b2RevoluteJoint_SetMotorSpeed(tj_motor_joint, 0.0);
	}
	
	if (key_d == 1 && tj_last_d == 0)
	{
		b2Body_SetAwake(tj_chassis, 1);
		b2RevoluteJoint_SetMotorSpeed(tj_motor_joint, tj_motor_speed);
	}
	
	if (key_m == 1 && tj_last_m == 0)
	{
		b2Body_SetAwake(tj_chassis, 1);
		b2RevoluteJoint_EnableMotor(tj_motor_joint, !(b2RevoluteJoint_IsMotorEnabled(tj_motor_joint)));
	}
	
	// Save key states
	tj_last_a = key_a;
	tj_last_s = key_s;
	tj_last_d = key_d;
	tj_last_m = key_m;
}

/// \brief Clean current test suite
void theojansen_exit()
{
	// Default clean
	deftest_exit(&theojansen_def_vars);
	
	// Show instruction panel
	tj_pan_instruction->flags &= ~SHOW;
}

/// \brief Create walker legs
///
/// @param s
/// @param wheel_anchor
void theojansen_create_leg(var s, VECTOR *wheel_anchor)
{
	DEF_VARIABLES *vars = &theojansen_def_vars;
	
	// vectors
	VECTOR p1; vec_set(&p1, vec2(162 * s / tj_scale, 182 / tj_scale));
	VECTOR p2; vec_set(&p2, vec2(216 * s / tj_scale, 36  / tj_scale));
	VECTOR p3; vec_set(&p3, vec2(129 * s / tj_scale, 57  / tj_scale));
	VECTOR p4; vec_set(&p4, vec2( 93 * s / tj_scale, -24  / tj_scale));
	VECTOR p5; vec_set(&p5, vec2(180 * s / tj_scale, -45  / tj_scale));
	VECTOR p6; vec_set(&p6, vec2( 75 * s / tj_scale, -111 / tj_scale));
	
	// Shape arrays
	VECTOR a1[3];
	VECTOR a2[3];
	VECTOR a3[3];
	VECTOR a4[3];
	
	// [p3, p2, p1]
	vec_set(&a1[0], &p3); vec_set(&a1[1], &p2); vec_set(&a1[2], &p1);
	// [p6-p4, p5-p4, 0]
	vec_set(&a2[0], &p6); vec_set(&a2[1], &p5); vec_set(&a2[2], vector(0, 0, 0));
	vec_sub(&a2[0], &p4); vec_sub(&a2[1], &p4);
	// [p2, p3, p1]
	vec_set(&a3[0], &p2); vec_set(&a3[1], &p3); vec_set(&a3[2], &p1);
	// [p5-p4, p6-p4, 0]
	vec_set(&a4[0], &p5); vec_set(&a4[1], &p6); vec_set(&a4[2], vector(0, 0, 0));
	vec_sub(&a4[0], &p4); vec_sub(&a4[1], &p4);
	
	b2PolygonShape *sd1 = b2PolygonShape_Create();
	b2PolygonShape *sd2 = b2PolygonShape_Create();
	b2FixtureDef *fd1 = b2FixtureDef_Create();
	b2FixtureDef *fd2 = b2FixtureDef_Create();
	
	b2FixtureDef_SetShape(fd1, sd1);
	b2FixtureDef_SetShape(fd2, sd2);
	
	b2Filter *filter = NULL;
	filter = b2FixtureDef_GetFilter(fd1);
	filter->groupIndex = -1;
	
	filter = b2FixtureDef_GetFilter(fd2);
	filter->groupIndex = -1;
	
	b2FixtureDef_SetDensity(fd1, 1.0);
	b2FixtureDef_SetDensity(fd2, 1.0);
	
	if (s > 0.0)
	{
		b2PolygonShape_Set(sd1, a1, 3);
		b2PolygonShape_Set(sd2, a2, 3);
	}
	else
	{
		b2PolygonShape_Set(sd1, a3, 3);
		b2PolygonShape_Set(sd2, a4, 3);
	}
	
	b2BodyDef *bd1 = b2BodyDef_Create();
	b2BodyDef_SetType(bd1, B2_DYNAMICBODY);
	
	b2BodyDef *bd2 = b2BodyDef_Create();
	b2BodyDef_SetType(bd2, B2_DYNAMICBODY);
	
	b2BodyDef_SetPosition(bd1, &tj_offset);
	
	VECTOR temp;
	vec_set(&temp, &p4);
	vec_add(&temp, &tj_offset);
	b2BodyDef_SetPosition(bd2, &temp);
	
	b2BodyDef_SetAngularDamping(bd1, 10.0);
	b2BodyDef_SetAngularDamping(bd2, 10.0);
	
	b2Body *body1 = b2World_CreateBody(vars->m_world, bd1);
	b2Body *body2 = b2World_CreateBody(vars->m_world, bd2);
	
	b2Body_CreateFixture(body1, fd1);
	b2Body_CreateFixture(body2, fd2);
	
	b2DistanceJointDef *djd = b2DistanceJointDef_Create();
	
	// Using a soft distance constraint can reduce some jitter.
	// It also makes the structure seem a bit more fluid by
	// acting like a suspension system.
	b2DistanceJointDef_SetDampingRatio(djd, 0.5);
	b2DistanceJointDef_SetFrequencyHz(djd, 10.0);
	
	b2DistanceJointDef_Initialize(djd, body1, body2, vec2(p2.x + tj_offset.x, p2.y + tj_offset.y), vec2(p5.x + tj_offset.x, p5.y + tj_offset.y));
	b2World_CreateJoint(vars->m_world, djd);
	
	b2DistanceJointDef_Initialize(djd, body1, body2, vec2(p3.x + tj_offset.x, p3.y + tj_offset.y), vec2(p4.x + tj_offset.x, p4.y + tj_offset.y));
	b2World_CreateJoint(vars->m_world, djd);
	
	b2DistanceJointDef_Initialize(djd, body1, tj_wheel, vec2(p3.x + tj_offset.x, p3.y + tj_offset.y), vec2(wheel_anchor.x + tj_offset.x, wheel_anchor.y + tj_offset.y));
	b2World_CreateJoint(vars->m_world, djd);
	
	b2DistanceJointDef_Initialize(djd, body2, tj_wheel, vec2(p6.x + tj_offset.x, p6.y + tj_offset.y), vec2(wheel_anchor.x + tj_offset.x, wheel_anchor.y + tj_offset.y));
	b2World_CreateJoint(vars->m_world, djd);
	
	b2RevoluteJointDef *rjd = b2RevoluteJointDef_Create();
	b2RevoluteJointDef_Initialize(rjd, body2, tj_chassis, vec2(p4.x + tj_offset.x, p4.y + tj_offset.y));
	b2World_CreateJoint(vars->m_world, rjd);
	
	// Destroy unused objects
	b2PolygonShape_Destroy(sd1);
	b2PolygonShape_Destroy(sd2);
	b2FixtureDef_Destroy(fd1);
	b2FixtureDef_Destroy(fd2);
	
	// bodydefs
	b2BodyDef_Destroy(bd1);
	
	b2BodyDef_Destroy(bd2);
	
	// joints
	b2DistanceJointDef_Destroy(djd);
	b2RevoluteJointDef_Destroy(rjd);
}

#endif // __TEST_THEO_JANSEN_H__
